class CustomerEndpointsController < RxModule::ApplicationController
  # This controller contains endpoints that are customer specific
  # When adding an endpoint here, please be specific at which site it is used.

  class RequestLMACRODLP
    @last_order_number = nil

    def execute(params)
      params = params.permit(:jobNumber)
      order_number = params[:jobNumber]

      return abort("Job missing from request params.", :bad_request) if order_number.nil?

      RequestLMACRODLP.log "Starting Request for Job #{order_number} | Previous Job: #{RequestLMACRODLP.last_order_number})"

      is_same_order = RequestLMACRODLP.got_same_request?(order_number)
      RequestLMACRODLP.set_last_order(order_number)

      RequestLMACRODLP.log "Not restarting job #{order_number} since it is the same as the last request." if is_same_order

      # Attempt to get the existing job so it is not restarted. If it doesn't exist treat it as a new job.
      order = nil
      order = Order.find_by_number(order_number) if is_same_order

      is_new_order = false
      if order.nil?
        order = Order.create_new(order_number)
        is_new_order = true
      end

      order.assign_attributes(is_routing: true)

      begin
        process_request = ProcessFetcher.fetch_for_job_number(order_number, Setting.instance.post_job_data? && is_new_order)
      rescue ProcessRequest::OrderDoesNotExist
        Router.instance.synchronize_model(order) do
          order.reject(Order.REJECT_REASONS[:failFetchProcesses], Time.zone.now, { reason: 'No data available for this job.' })
          order.save!
        end
        return abort("Job does not exist in data source.", :not_found)
      rescue => err
        Router.instance.synchronize_model(order) do
          order.reject(Order.REJECT_REASONS[:failFetchProcesses], Time.zone.now, { reason: 'Failed to fetch data from the LMS for this job.' })
          order.save!
        end
        return abort("Got no response from data source.", :service_unavailable)
      end

      if is_new_order
        order.set_rx_data_identifiers(process_request.rx_data_order_id, process_request.rx_data_job_ids, process_request.lens_numbers)
        order.set_required_processes(process_request.processes)
        order.save!
      end

      macro_data = extract_lmacrodlp(process_request.tags)

      if macro_data.nil?
        Router.instance.synchronize_model(order) do
          order.reject(Order.REJECT_REASONS[:receivedStartInReject])
          order.save!
        end
        return abort("Tag response did not contain LMACRODLP.", :not_found)
      end

      Router.instance.synchronize_model(order) do
        order.save!
      end

      response(order.number, macro_data)
    end

    def abort(error, status)
      RequestLMACRODLP.log "Call Aborted. Error: #{error}"
      {
          error: error,
          status: status
      }
    end

    def extract_lmacrodlp(tags)
      left_macro = extract_lmacrodlp_for_side(tags[:left])
      right_macro = extract_lmacrodlp_for_side(tags[:right])

      return nil if left_macro.nil? && right_macro.nil?

      # If the LMACRO is only provided for one side, use it for both sides
      return "#{right_macro};#{right_macro}" if left_macro.blank?
      return "#{left_macro};#{left_macro}" if right_macro.blank?

      "#{right_macro};#{left_macro}"
    end

    def extract_lmacrodlp_for_side(side_tags)
      return nil if side_tags.nil?

      lmacro = side_tags["_LMACRODLP"]
      lmacro = side_tags["LMACRODLP"] if lmacro.nil?
      lmacro = side_tags["MES_LMACRODLP"] if lmacro.nil?
      lmacro = side_tags["MES__LMACRODLP"] if lmacro.nil?

      lmacro
    end

    def response(order_number, macro_data)
      require 'nokogiri'

      resp = Nokogiri::XML::Builder.new(:encoding => 'UTF-8') do |xml|
        xml.LMACRODLP macro_data
      end

      RequestLMACRODLP.log "Response to LMACRODLP call for job #{order_number} => #{resp.to_xml}"

      {status: :ok, data: resp}
    end

    def self.set_last_order(number)
      @last_order_number = number
    end

    def self.last_order_number
      @last_order_number
    end

    def self.got_same_request?(order_number)
      return false if @last_order_number.nil?
      @last_order_number == order_number
    end

    private

    def self.log(message)
      Rails::logger.info "[Request LMACRODLP] #{message}"
    end
  end

  # This endpoint is used at the Nikon Aichi Lab. On their line, an RFID reader must first request the LMACRODLP data from the LMS before a job can be routed.
  # This also servers as the hook to create a a job in the router (setting must be set to 'on_external')
  def lmacrodlp_scan
    resp = RequestLMACRODLP.new.execute(params)

    if resp[:status] == :ok
      render xml: resp[:data]
    else
      render json: {}, status: resp[:status]
      # Todo: right now we're just going to render an empty JSON since we don't know how the system will react to JSON data
      #render json: {error: resp[:error]}, status: resp[:status]
    end
  end
end
