class MachinesController < RxModule::ApplicationController
  include PushDataResponse
  include UseComponentName

  def self.set_target_before
    super + [:set_machine_virtual]
  end

  model_data_store(Machine)
  also_protect_with([
                      RxModule::Authorization.verb("OverwriteType", "Overwrite Type", :set_machine_virtual)
                    ])

  def before_set_targets
    # Fetch Rx-Data machines if it's the first time loading - otherwise Rx-Data machines will never be loaded
    MachineFetcher.reload_cache if MachineFetcher.force_cache_reload?

    # both left_outer_joins because we still want machines without job_process or capabilities
    # sub with device classname and the name of the component relationship
    models = Machine.joins(:component).left_outer_joins(:job_process).left_outer_joins(:capabilities).distinct

    return models if params['filtered'].nil?

    # we have a breadModel for only mes machines
    if params['filtered'].any? {|filter| filter[:id] === 'rx_data_id' && filter[:value] === 'is not null'}
      params['filtered'] = params['filtered'].select {|filter| !(filter[:id] === 'rx_data_id' && filter[:value] === 'is not null')}
      models.where.not(rx_data_id: nil).where.not(type: 'MachineVirtual')
    elsif params['filtered'].any? {|filter| filter[:id] === 'rx_data_id' && filter[:value] === 'is null'}
      params['filtered'] = params['filtered'].select {|filter| !(filter[:id] === 'rx_data_id' && filter[:value] === 'is null')}
      models.where(rx_data_id: nil).or(models.where.not(rx_data_id: nil).where(type: 'MachineVirtual'))
    end
  end

  def before_filter(filter)
    if filter[:id] == 'name'
      filter[:id] = 'components.name'
    elsif filter[:id] == 'job_process'
      filter[:id] = 'job_process.name'
    elsif filter[:id] == 'capability_ids'
      filter[:id] = 'capabilities.rank'
    end
    filter
  end

  def before_sort(filter)
    if filter[:id] == 'name'
      filter[:id] = 'components.name'
    elsif filter[:id] == 'job_process'
      filter[:id] = 'job_process.name'
    elsif filter[:id] == 'capability_ids'
      filter[:id] = 'capabilities.name'
    end
    filter
  end

  def create
    params = model_params

    @model = Machine.machine_template_to_class[params[:type]].new_from_params(params)

    if @model.save
      render json: @model
    else
      render json: @model.errors, status: :unprocessable_entity
    end
  end

  def update
    params = model_params

=begin
# MACHINE BULK UPDATE DISABLED AS OF 2024-01-07
# two requirements to make this work:
#  a) needs to be a selector (machines can have several groups
#  b) disabled while the router is online
    begin
      machines_to_update = params[:update_machine_group] ? @model.machines_in_same_group : [@model]
      machines_to_update.each do |machine|
        machine.update_from_params(params)
      end
    rescue
      return render json: {error: "Something went wrong while updating the machine group."}, status: :internal_server_error
    end
=end
    Router.instance.synchronize_model(@model) do
      @model.ensure_order_machine_schedule(params[:job_process_id])
      @model.assign_attributes_from_params(params)
      @model.save!
    end

    render :show
  end

  def destroy
    if @model.destroy
      render json: @model
    else
      render json: @model.errors, status: :unprocessable_entity
    end
  end

  def reject_stations
    failed = false
    processed_job_process = JobProcess.find_by_name('processed')
    if processed_job_process.nil?
      render json: nil
      return
    end
    rejected_capability = Capability.find_by_name('rejected')
    if rejected_capability.nil?
      render json: nil
      return
    end
    process_id = processed_job_process[:id]
    capability_id = rejected_capability[:id]
    processed_machines = Machine.where(job_process_id: process_id)
    rejected_machines = []
    processed_machines.each do |machine|
      if machine.capability_ids.include? capability_id
        rejected_machines << {
            id: machine[:id],
            name: machine.component[:name]
        }
      end
    end
    render json: rejected_machines
  end

  def set_machine_virtual
    @model.update!({ type: 'MachineVirtual', apply_process_change_on: 'auto_finish', forced_state: 'online' })
    render :show
  end

  # A machine was created/updated/deleted in rx-data
  def change_from_external
    params = rx_data_change_params

    #If changes is for a non conveyor machine in rx-data, ignore
    return send_push_response unless params[:is_conveyor_machine]

    if params[:destroyed] == true
      machine = MachineMes.find_by_rx_data_id(params[:id])
      unless machine.nil?
        Rails.logger.warn "Machine #{machine.component[:name]} was deleted in rx-data."
        #TODO: this should probably destroy the machine here too?
      end
    else
      if MachineMes.exists?(sn: params[:sn])
        machine = MachineMes.find_by_sn(params[:sn])
        machine.component[:name] = params[:name]
        machine.save
      else
        MachineMes.new_from_params(params.merge({ rx_data_id: params[:id]}))
      end
    end

    send_push_response
  end

  def state_from_external
    MachineFetcher.state_update_from_external(rx_data_state_params)
    send_push_response
  end

  # Legacy endpoint to support an S4P endpoint that allowed machines to toggle capabilities.
  # This is in use at the Pionneer Lab in Asia
  def capability_update
    resp = {status: 'updated'}

    if params[:machine_sn].nil? || params[:capabilities].nil?
      Rails::logger.warn "Failed to update machine capabilities, missing params."
      return render json: resp, status: :ok
    end

    @model = MachineMes.find_by_sn(params[:machine_sn])
    if @model.nil?
      Rails::logger.warn "Failed to update machine capabilities for machine SN #{params[:machine_sn]}. No such machine."
      return render json: resp, status: :ok
    end

    Router.instance.synchronize_model(@model) do
      enabled_capabilities = params[:capabilities].map {|c| c[:name]}

      enabled = []
      disabled = []
      @model.capabilities.each do |capability|
        if enabled_capabilities.include? capability.name
          enabled << capability.id
        else
          disabled << capability.id
        end
      end

      @model.capability_machines.where(capability_id: enabled).update_all(enabled: true) unless enabled.empty?
      @model.capability_machines.where(capability_id: disabled).update_all(enabled: false) unless disabled.empty?
    end

    render json: resp, status: :ok
  end

  def to_json(*args)
    render_jbuilder_partial
  end

  private

  def model_params
    params.fetch(:machine).permit( :id,
                                   :type,
                                   :name,
                                   :force_duration,
                                   :job_process_id,
                                   :state,
                                   :update_machine_group,
                                   :selected_reject,
                                   :forced_state,
                                   :apply_process_change_on,
                                   :allow_job_start,
                                   :restart_jobs,
                                   :lms_config_id,
                                   :capability_ids => [],
                                   :capabilities_to_add => [:capability_id],
                                   :mes_machine_states_to_add => [:name, :online])
  end

  def rx_data_change_params
    if params[:destroyed] == true
      params.permit( :id, :destroyed)
    else
      params.permit( :id, :sn, :name, :description, :is_conveyor_machine, :action, :machine_group_ids => [])
    end
  end

  def rx_data_state_params
    params.permit( :state, :machine_id, :time, :machine_sn)
  end
end
