class OrdersController < RxModule::ApplicationController
  include RouterResponse
  include RouterDataWrapper
  include PushDataResponse

  def self.set_target_before
    super + [:complete_up_to_process, :reset_down_to_process, :rework_up_to_process, :update_required_processes, :toggle_reject, :status, :set_order_machine_schedule]
  end

  model_data_store(Order)
  also_protect_with([
                      RxModule::Authorization.verb("ChangeProcesses", "Change Processes", :update_required_processes),
                      RxModule::Authorization.verb("MachineSchedule", "Machine Schedule", :set_order_machine_schedule),
                      RxModule::Authorization.verb("ReworkProcess", "Rework Process", :rework_up_to_process)
                    ])

  def create
      order_number = create_params[:order_number]
      @model = OrderRequest::OrderRequest.force_new_order(order_number)
      @model.update(is_routing: true)
      @model.send_router_create
      render json: @model, status: :created, location: @model
  end

  def destroy
    @model.ensure_destroy!
    head :no_content
  rescue => err
    render json: { error: err }, status: :conflict
  end

  def search
    @model = Order.find_by_number(search_params[:order_number])
    return render json: {error: "JobDoesNotExist"}, status: :not_found if @model.nil?
    render :show
  end

  def search_data
    job_number = search_data_params[:order_number]

    return render json: {error: "Invalid Job Number"}, status: :unprocessable_entity if job_number.blank?

    process_data = ProcessFetcher.fetch_for_job_number(job_number)

    if process_data.job_active
      render json: {
        job_number: process_data.job_number,
        tags: process_data.tags,
        processes: process_data.processes,
        source: process_data.source
      }
    else
      render json: {error: "Job is inactive."}, status: :unprocessable_entity
    end
  rescue ProcessRequest::OrderDoesNotExist
    render json: {error: "Job is archived or does not exist."}, status: :not_found
  rescue ProcessRequest::FailedToFetchFromSource
    render json: {error: "Failed to fetch from tag source."}, status: :internal_server_error
  rescue => err
    render json: {error: err}, status: :unprocessable_entity
  ensure
    # Force the Job Process cache to flush if the router isn't on, other wise settings change would not be testable
    JobProcessCalculator.instance.clear_cache! unless Router.instance.running
  end

  def status
    status = OrderStatusFetcher.order_status(@model.rx_data_order_id)
    render json: status, status: :ok
  rescue => err
    render json: {error: 'Could not fetch order status'}, status: :unprocessable_entity
  end

  def set_order_machine_schedule
    params = set_order_machine_schedule_params
    new_machine_ids = params[:machine_ids]

    Router.instance.synchronize_model(@model) do
      @model.assign_order_machine_schedule(new_machine_ids)

      @model.save_and_force_refresh!
    end

    render :show
  end

  concerning :RequiredProcessActions do
    def complete_up_to_process
      params = required_process_action_params

      Router.instance.synchronize_model(@model) do
        @model.complete_processes_up_to(params[:job_process_rank])
        @model.save!
      end

      render :show
    end

    def reset_down_to_process
      params = required_process_action_params

      Router.instance.synchronize_model(@model) do
        @model.reset_processes_down_to(params[:job_process_rank])
        @model.save!
      end

      render :show
    end

    def rework_up_to_process
      params = required_process_rework_params

      Router.instance.synchronize_model(@model) do
        @model.rework_processes_up_to(params[:job_process_rank], params[:side])
        @model.save!
      end

      render :show
    end

    def update_required_processes
      params = update_required_processes_params

      Router.instance.synchronize_model(@model) do
        Order.transaction do
          @model.update_required_processes(params[:required_processes_data])
          @model.save_and_force_refresh!
        end
      end

      render :show
    end

    def toggle_reject
      Router.instance.synchronize_model(@model) do
        if @model.rejected?
          @model.unreject
        else
          @model.reject(Order.REJECT_REASONS[:manual])
        end
        @model.save!
      end

      render :show
    end

    private

    def required_process_action_params
      params.permit(:job_process_rank)
    end

    def required_process_rework_params
      params.permit(:job_process_rank, :side)
    end

    def update_required_processes_params
      params.permit(:required_processes_data => [:job_process_id, :left => {:capabilities => []}, :right => {:capabilities => []}])
    end
  end

  concerning :RouterRoutes do
    def fetch
      params = router_fetch_params
      order = OrderRequest::OrderRequest.order_for_number(params[:jobNumber], params[:requestType])
      render json: order.as_router_data, status: :ok
    end

    def update_processes
      params = update_processes_params
      RouterEndpoints::UpdateProcessesJob.perform_later(params)
      send_router_response
    end

    def update_location
      params = update_location_params
      RouterEndpoints::UpdateLocationJob.perform_later(params)
      send_router_response
    end

    def update_reject
      params = update_reject_params
      RouterEndpoints::UpdateRejectJob.perform_later(params)
      send_router_response
    end

    def complete
      params = complete_params
      RouterEndpoints::CompleteJob.perform_later(params)
      send_router_response
    end

    def schedule_process_state_change
      RouterEndpoints::ScheduleProcessStateChangeJob.perform_later(schedule_process_state_change_params)
      send_router_response
    end

    def add_process
      params = add_process_params

      order = Order.find_by(number: params[:order])
      process_id = JobProcess.where(name: params[:process]).pluck(:id)[0]

      return render json: {error: 'Order does not exist'}, status: :unprocessable_entity if order.nil?
      return render json: {error: 'Process does not exist'}, status: :unprocessable_entity if process_id.nil?

      unless order.has_process_named?(params[:process])
        %w[left right].each do |side|
          job = order.job_for_side(side)
          next unless job.required?
          job.add_process(process_id, {capabilities: []})
        end
        order.save!
      end

      render json: order.as_router_data, status: :ok
    end

    private
    def router_fetch_params
      dig_to_data.permit(:jobNumber, :requestType)
    end

    def update_processes_params
      dig_to_data.permit(:order, :processes => [:id, :state, :time, :source, :lastMachineComment, :logToHistory, :isRework])
    end

    def update_location_params
      dig_to_data.permit(:order, :locations => [:location, :time])
    end

    def update_reject_params
      dig_to_data.permit(:order, :isReject, :time, rejectData: {})
    end

    def complete_params
      dig_to_data.permit(:order)
    end

    def schedule_process_state_change_params
      dig_to_data.permit(:order, :machine)
    end

    def add_process_params
      dig_to_data.permit(:order, :process)
    end
  end

  concerning :RxNet do

    def update_from_rx_net
      order_params = update_from_rx_net_params

      sides = get_sides_from_tag_data(order_params[:tag_data])
      return send_push_response if sides.nil?

      order = get_order_from_job_numbers(order_params[:job_numbers])
      return send_push_response if order.nil?

      required_process_data = JobProcessCalculator.instance.get_update_required_processes_data_from_tags(order_params[:tag_data], sides)

      Router.instance.synchronize_model(order) do
        order.apply_update_from_rx_net(required_process_data)
      end

      send_push_response
    end

    private

    def get_sides_from_tag_data(tag_data)
      if tag_data.blank?
        Rails.logger.error "Missing tag data."
        return nil
      end

      sides = []
      sides << :right unless tag_data[:right].nil?
      sides << :left unless tag_data[:left].nil?
      sides
    end

    def get_order_from_job_numbers(job_numbers)

      # NOTE: Typically, when getting routing_launch_order data, Rx-Routing sends the request, with the number column pre-determined
      # In this case, since this request came from Rx-Data directly, the number column is not determined and has to be assumed to be the same as job_numbers

      if job_numbers.blank?
        Rails.logger.error "No job numbers provided."
        return nil
      end

      job_models = []

      job_numbers.each do |job_info|
        job = Job.find_by(number: job_info[:jobNumber], side: job_info[:lens])
        next if job.nil?
        job_models << job
      end

      if job_models.blank?
        Rails.logger.error "Cannot Update Order - Job #{job_numbers.map { |job_info| "#{job_info[:lens]}: #{job_info[:jobNumber]}" }.join(' | ')} does not exist."
        return nil
      end

      order_ids = job_models.map { |job_model| job_model.order.id }.uniq
      unless order_ids.size == 1
        Rails.logger.error "Cannot Update Order - Job #{job_numbers.map { |job_info| "#{job_info[:lens]}: #{job_info[:jobNumber]}" }.join(' and ')} belong to different orders."
        return nil
      end

      order = Order.find_by(id: order_ids.first)
      if order.nil?
        Rails.logger.error "Could not find order."
        return nil
      end

      order
    end

    def update_from_rx_net_params
      params.permit(:job_numbers => [:jobNumber, :lens], tag_data: {left: {}, right: {}}).to_h
    end
  end

  private

  def create_params
    params.permit(:order_number)
  end

  def search_params
    params.permit(:order_number)
  end

  def search_data_params
    params.permit( :order_number)
  end

  def set_order_machine_schedule_params
    params.permit(:machine_ids => [])
  end
end