class RoutersController < RxModule::ApplicationController
  include RouterResponse
  include RouterDataWrapper

  def self.set_target_before
    super + [:start, :stop, :state]
  end

  def set_target
    @model = Router.instance
  end

  # Index action is required for the front-end store to be able to load the singleton model
  model_data_store(Router,
                   {create: false, read: true, update: false, delete: false, index: true},
                   {create: false, read: true, update: false, delete: false, index: false})
  also_protect_with([
                      RxModule::Authorization.verb("start", "start", :start),
                      RxModule::Authorization.verb("stop", "stop", :stop)
                    ])

  def start
    @model.start
    @model.update(status: 'starting', time: Time.zone.now)
    render :show
  rescue => err
    render json: {error: err}, status: :unprocessable_entity
  end

  def stop
    @model.stop
    @model.update(status: 'offline', time: Time.zone.now)
    render :show
  rescue => err
    render json: {error: err}, status: :unprocessable_entity
  end

  concerning :RouterStartup do
    def communicators
      render json: {
        communicators: Setting.instance.communicator_settings
      }
    end

    def base_settings
      render json: {
        options: Setting.instance.base_settings
      }
    end

    def plc_settings
      render json: {
        options: Setting.instance.plc_settings
      }
    end

    def device_definitions
      render json: {
        definitions: DeviceType.all.map do |device_type|
          device_type.get_json
        end
      }
    end

    def devices
      render json: {
        devices: Device.all.map do |device|
          device.as_router_data
        end
      }
    end

    def machines
      render json: {
        machines: RoutingGraph.instance.routing_machines.map do |machine|
          machine.as_router_data
        end
      }
    end

    def processes
      render json: {
        processes: RoutingGraph.instance.processes_offered.filter {|process| !process.is_control_process }.map do |process|
          process.as_router_data
        end
      }
    end

    def routes
      render json:  {
        routes: RoutingGraph.instance.as_router_data
      }
    end

    def default_job_processes
      render json: {
        processes: ProcessFetcher.default_processes_json
      }
    end

    def settings
      render json: {
        settings: Setting.instance.as_router_data
      }
    end
  end

  # Routes used by the router itself
  concerning :RouterOperational do
    def systems_status
      render json: ExternalSystems.statuses, status: :ok
    end

    def refresh_machine_statuses
      MachineFetcher.reload_cache(true)
      render json: {
        machines: RoutingGraph.instance.routing_machines.map do |machine|
          attributes = machine.external_attributes
          {
            identifier: machine.name,
            status: {
              state: attributes[:state],
              time: attributes[:time]
            }
          }
        end
      }
    end

    def state
      params = state_params
      time = Time.zone.parse(params[:time])

      if @model.time <= time
        @model.update(status: params[:status], time: time)
      end

      send_router_response
    end

    def events
      params = event_params
      event = Event.new(params)
      unless event.save
        Rails::logger.warn "Failed to create Event, error: #{event.errors}"
      end

      send_router_response
    end

    private
    def state_params
      dig_to_data.permit(:status, :time)
    end

    def event_params
      p = dig_to_data.fetch(:event).permit(:time, :eventType, :message, details: {})
      {
        time: p[:time],
        event_type: p[:eventType],
        message: p[:message],
        details: p[:details].nil? ? "{}" : p[:details].to_json
      }
    end
  end
end

