import * as React from "react";
import {observer} from "mobx-react-lite";
import ReactTable, {Column, TableProps} from "react-table";
import {useCallback, useEffect, useMemo, useState} from "react";
import {DataStore, FetchParams, Model} from "@rx-suite/data";
import {LatestModelsStore} from "../stores/LatestModelsStore";
import {CircularProgress} from "@material-ui/core";
import {useFilteringDebounce} from "../hooks/useFilteringDebounce";

type FirstInFirstOutTableProps = {
    store: DataStore<Model>,
    channelName: string,
    columns: Column[],
    sortingColumn?: string
} & Partial<Omit<TableProps, "data" | "loading" | "onPageSizeChange" | "onPageChange" | "manual">>

export const LatestModelsTable: React.FunctionComponent<FirstInFirstOutTableProps> = observer(({
                                                                                                   store,
                                                                                                   channelName,
                                                                                                   columns,
                                                                                                   sortingColumn,
                                                                                                   ...props
}) => {
    const [fetchParams, setFetchParams] = useState<FetchParams>({
        limit: 10,
        sorted: [{id: sortingColumn || 'created_at', desc: true}],
        filtered: [],
        page: 0
    });
    const [latestModelsStore, setLatestModelsStore] = useState<LatestModelsStore | undefined>(undefined)

    const fetch = useCallback((state) => {
        setFetchParams(prevState => {
            return {
                ...prevState,
                filtered: state.filtered
            }
        })
    }, [setFetchParams])

    const [handleFilteredChange, fetchDataStrategy] = useFilteringDebounce(!latestModelsStore?.fetching, fetch);

    useEffect(() => {
        const s = new LatestModelsStore(store, channelName, fetchParams)
        setLatestModelsStore(s);

        return () => {
            s.destructor();
        }
    }, [fetchParams]);

    const maxPageNumber = useMemo(
        () => Math.ceil((latestModelsStore?.count || 0) / (fetchParams.limit || 1)),
        [latestModelsStore?.count, fetchParams.limit]
    );

    const setPageSize = useCallback((number: number) => {
        setFetchParams(prevState => {
            return {
                ...prevState,
                limit: number
            }
        })
    }, [setFetchParams])

    const setPageNumber = useCallback((number: number) => {
        setFetchParams(prevState => {
            return {
                ...prevState,
                page: number
            }
        })
    }, [setFetchParams])

    if(!latestModelsStore) {
        return<CircularProgress/>
    }

    return <ReactTable
        data={latestModelsStore.models}
        columns={columns}
        loading={latestModelsStore.fetching}
        pageSize={fetchParams.limit}
        page={fetchParams.page}
        onPageSizeChange={setPageSize}
        onPageChange={setPageNumber}
        pages={maxPageNumber}
        manual={true}
        onFilteredChange={handleFilteredChange}
        onFetchData={fetchDataStrategy}
        resolveData={data => data.map(r => r || Model)}
        {...props}
    />
})