import * as React from "react";
import {observer} from "mobx-react-lite";
import {EventModel, EventStore} from "../../stores/EventStore";
import {ButtonField, makeColumn, SelectField} from "@rx-suite/ui";
import {useMemo, useState} from "react";
import {t} from "@lingui/macro";
import {LatestModelsTable} from "../LatestModelsTable";
import {SystemSettingModel, SystemSettingStore} from "@rx-suite/stores";
import {Filter, ReactTableFunction} from "react-table";
import {Moment} from "moment";
import {createStyles, Theme, Typography} from "@material-ui/core";
import {makeStyles} from "@material-ui/core/styles";
import {blue, grey, orange, red} from "@material-ui/core/colors";
import {EventDetailsDialog} from "./EventDetailsDialog";

const useStyles = makeStyles((theme: Theme) =>
    createStyles({
        header: {
            fontWeight: 'bold',
            fontSize: '1.5rem',
            color: grey[700],
            marginBottom: '24px'
        },
        detailsButton: {
            margin: '0px'
        },
        yellow: {
            color: orange[500]
        },
        blue: {
            color: blue[500]
        },
        red: {
            color: red[500]
        }
    })
);

export const EventsTable: React.FunctionComponent = observer(() => {
    const classes = useStyles();
    const systemSettings = SystemSettingStore.selectedModel as SystemSettingModel;

    const [eventForDetails, setEventForDetails] = useState<EventModel | null>(null);

    const eventTypeOptions = [
        {label: 'All', value: ''},
        {label: 'info', value: 'info'},
        {label: 'warning', value: 'warning'},
        {label: 'fatal', value: 'fatal'}
    ]

    const colorRouter = (eventType: string) => {
        switch(eventType){
            case 'info':
                return 'blue';
            case 'warning':
                return 'yellow';
            case 'fatal':
                return 'red'
        }
        return 'blue'
    }

    const columns = useMemo(() => {
        return [
            {
                Header: t`Time`,
                accessor: 'time',
                width: 175,
                Cell: (cellInfo: { value: Moment }) => {
                    return systemSettings.printDateTime(cellInfo.value);
                },
                filterable: false,
                sortable: false
            },
            {
                Header: t`Event Type`,
                accessor: 'event_type',
                width: 110,
                Cell: (cellInfo: { value: string }) => {
                    return <Typography className={classes[colorRouter(cellInfo.value)]}>{cellInfo.value}</Typography>
                },
                filterable: true,
                Filter: ({filter, onChange}: { filter: Filter, onChange: ReactTableFunction }) => {
                    return <SelectField
                        options={eventTypeOptions}
                        onChange={(name, value) => onChange(value)}
                        value={filter ? filter.value : ""}
                    />
                }
            },
            makeColumn('message', t`Message`, 'string'),
            {
                Header: t`Actions`,
                accessor: (event: EventModel) => event,
                id: '_actions',
                width: 110,
                Cell: (cellInfo: { value: EventModel }) => {
                    return <ButtonField
                        variant={'text'}
                        text={<Typography style={{textDecoration: 'underline', textTransform: 'none'}}>{t`view details`}</Typography>}
                        className={classes.detailsButton}
                        onClick={() => setEventForDetails(cellInfo.value)}
                    />
                },
            }
        ]
    }, [])

    return <React.Fragment>
        <Typography className={classes.header}>{t`Events`}</Typography>
        <LatestModelsTable
            columns={columns}
            store={EventStore}
            channelName={'EventsChannel'}
        />
        <EventDetailsDialog model={eventForDetails} setModel={setEventForDetails}/>
    </React.Fragment>
})
