import * as React from "react";
import {observer} from "mobx-react-lite";
import {System, useExternalSystems} from "../../hooks/useExternalSystems";
import {Box, CircularProgress, createStyles, Grid, Theme, Typography} from "@material-ui/core";
import {t} from "@lingui/macro";
import {makeStyles} from "@material-ui/core/styles";
import ReactTable, {Column} from "react-table";
import {useMemo} from "react";
import {green, grey, red} from "@material-ui/core/colors";
import classNames from "classnames";
import {currentUserCan, SystemSettingModel, SystemSettingStore} from "@rx-suite/stores";
import {LmsReinitializeButton} from "./LmsReinitializeButton";
import {LmsConfigStore} from "../../stores/LmsConfigStore";

const useStyles = makeStyles((theme: Theme) =>
    createStyles({
        header: {
            fontWeight: 'bold',
            fontSize: '1.5rem',
            color: grey[700]
        },
        statusText: {
            fontWeight: 'bold'
        },
        grey: {
            color: grey[500]
        },
        green: {
            color: green[500]
        },
        red: {
            color: red[500]
        }
    })
);

export const ExternalSystemsStatus: React.FunctionComponent = observer(() => {
    const classes = useStyles();
    const [externalSystems, statusLoading] = useExternalSystems();
    const systemSettings = SystemSettingStore.selectedModel as SystemSettingModel;

    const userCanReinitLms = currentUserCan(LmsConfigStore.permissionTarget, 'reinitialize');

    const columns =  useMemo((): Column[] => {
        let cols = [
            {
                Header: t`Name`,
                accessor: 'name'
            },
            {
                Header: t`Status`,
                accessor: 'online',
                Cell: (cellInfo: { value: boolean }) => {
                    if(cellInfo.value) {
                        return <Typography className={classNames(classes.green, classes.statusText)}>{t`online`}</Typography>
                    }
                    return <Typography className={classNames(classes.red, classes.statusText)}>{t`offline`}</Typography>
                }
            }
        ];

        if(userCanReinitLms && externalSystems.systems.some(system => system.lms_config_id !== undefined)){
            cols.push({
                Header: t`Actions`,
                accessor: (system: System) => system,
                id: 'actions',
                Cell: (cellInfo: { value: System }) => {
                    if(!cellInfo.value.lms_config_id) { return null; }

                    return (
                        <Grid container justifyContent={'center'}>
                            <LmsReinitializeButton lmsConfig={cellInfo.value}/>
                        </Grid>
                    )
                }
            })
        }

        return cols;
    }, [externalSystems.systems, userCanReinitLms])

    const statusTable = useMemo(() => {
        const resolvedData = externalSystems.systems;

        if(statusLoading || !systemSettings) {
            return (
                <Box mt={2}>
                    <CircularProgress/>
                    <Box>
                        <Typography variant={'caption'}>{t`Fetching System Status...`}</Typography>
                    </Box>
                </Box>
            )
        }

        return <ReactTable
            data={resolvedData}
            columns={columns}
            pageSize={externalSystems.systems.length > 5 ? externalSystems.systems.length : 5}
            showPagination={false}
        />
    }, [externalSystems.systems, systemSettings, statusLoading])

    return <div>
        <Typography className={classes.header}>{t`System Status`}</Typography>
        <Typography className={classes.grey}>{t`Updated at` + ': ' + systemSettings.printDateTime(externalSystems.updated_at)}</Typography>
        {statusTable}
    </div>
})