import React from "react";
import {Conditional, HintButton, ModelCheckbox, ModelSelect, useModelContext} from "@rx-suite/ui";
import {observer} from "mobx-react-lite";
import {Box, Grid, makeStyles, Typography} from "@material-ui/core";
import {t} from "@lingui/macro";
import {HintList} from "../../../../generic/HintList";
import {ConnectorModel} from "../../../../../stores/ConnectorStore";
import {useProcessSelectByName} from "../../../../../hooks/useProcessSelectByName";
import {JobProcessStore} from "../../../../../stores/machine/JobProcessStore";
import {CapabilityStore} from "../../../../../stores/machine/CapabilityStore";

const useStyles = makeStyles((theme) => ({
     sectionHeader: {
         fontSize: '1.15rem',
         fontWeight: 'bold'
     }
 }))

export const ControlActionModifyJobDataForm: React.FunctionComponent = observer(() => {
    const model = useModelContext() as ConnectorModel;
    const classes = useStyles();

    const modifyJobsHint =  [
        {title: t`Unreject`, text: t`Jobs that arrive at this input and that are rejected will be unrejected.`},
        {title: t`Set Process`, text: t`Jobs that arrive at this input will have the selected process set to 'finished' or 'notstarted'.`},
        {title: t`Control Process`, text: t`Jobs that arrive at this input with the control process in the aborted state will have the selected process added to their process list (i.e:  add inspection if generating aborted).`}
    ];

    const forceToProcessHint =  [
        {title: t`Finish Previous Processes`, text: t`Processes prior to the selected process will be set to 'finished'.`},
        {title: t`Restart Next Processes`, text: t`Processes after the selected process will be set to 'notstarted'.`}
    ];

    const addControlProcessHint =  [
        {title: t`Process to Inject`, text: t`The process to add to the job when the control process is in the 'aborted' state.`},
        {title: t`Filter Process`, text: t`(optional) The control process will only be added if the Job already contains the filter process (i.e: only add inspection if job also has chamfering).`}
    ];

    const forceProcessStateOptions = [
        {label: t`Finished`, value: "finished"},
        {label: t`Not Started`, value: "notstarted"}
    ]

    const allProcessSelect = useProcessSelectByName();

    const processCapabilities = React.useMemo(()=> {
        const processName = model.viewModel.control_action_config?.addControlProcess?.filterProcess as string;

        if(JobProcessStore.isLoading || CapabilityStore.isLoading || !model.viewModel.control_action_config?.addControlProcess?.filterProcess){
            return [];
        }

        const process = JobProcessStore.models.find(process => process.name === processName);

        if(!process){
            return [];
        }

        return CapabilityStore.models
            .filter(capability => capability.job_process_id === process.id)
            .map(capability => ({label: capability.resolvedName, value: capability.name}));
    }, [JobProcessStore.models, CapabilityStore.models, model.viewModel.control_action_config?.addControlProcess?.filterProcess])

    return (
        <React.Fragment>
            <Box mb={-1.5}>
                <Grid container alignItems={'center'}>
                    <Typography className={classes.sectionHeader}>{t`Modify Job Data`}</Typography>
                    <HintButton
                        title={t`Modify Job Data`}
                        children={<HintList
                            primaryText={t`Allows the input to modify the job data.`}
                            items={modifyJobsHint}/>
                        }
                    />
                </Grid>
            </Box>
            <Box>
                <ModelCheckbox label={t`Unreject Jobs`} name={'control_action_config.unreject'}/>
            </Box>
            <Box>
                <ModelSelect
                    name={'control_action_config.setProcesses.process'}
                    label={'Set Process'}
                    options={allProcessSelect}
                    hint={
                        {
                            title: t`Set Process`,
                            children: <HintList items={forceToProcessHint}/>
                        }
                    }
                />
                <Conditional condition={model.forceToProcessEnabled}>
                    <Box ml={1.75}>
                        <Box>
                            <ModelSelect
                                name={'control_action_config.setProcesses.setToState'}
                                label={t`Set to State`}
                                options={forceProcessStateOptions}
                            />
                        </Box>
                        <Box mt={-0.5}>
                            <ModelCheckbox label={t`Finish Previous Processes`} name={'control_action_config.setProcesses.finishPrevious'}/>
                        </Box>
                        <Box mt={-1.5}>
                            <ModelCheckbox label={t`Restart Next Processes`} name={'control_action_config.setProcesses.restartNext'}/>
                        </Box>
                    </Box>
                </Conditional>
            </Box>
            <Box>
                <ModelSelect
                    name={'control_action_config.addControlProcess.process'}
                    label={'Control Process'}
                    options={allProcessSelect}
                    hint={
                        {
                            title: t`Control Process`,
                            children: <HintList items={addControlProcessHint}/>
                        }
                    }
                />
                <Conditional condition={model.addControlProcessEnabled}>
                    <Box ml={1.75}>
                        <ModelSelect
                            name={'control_action_config.addControlProcess.processToInject'}
                            label={'Process to Inject'}
                            options={allProcessSelect}
                        />
                        <ModelSelect
                            name={'control_action_config.addControlProcess.filterProcess'}
                            label={'Filter Process'}
                            options={allProcessSelect}
                        />
                        <Conditional condition={processCapabilities.length > 0}>
                            <ModelSelect
                                name={'control_action_config.addControlProcess.filterCapabilities'}
                                label={'Filter Capabilities'}
                                options={processCapabilities}
                                multiple
                            />
                        </Conditional>
                    </Box>
                </Conditional>
            </Box>
        </React.Fragment>
    )
});