import React from "react";
import {observer} from "mobx-react-lite";
import {
    createStyles,
    Dialog,
    DialogTitle,
    Theme,
    Box,
    DialogActions,
    DialogContent,
    Grid,
    Checkbox,
    Typography,
    Divider
} from "@material-ui/core";
import {t} from "@lingui/macro";
import {makeStyles} from "@material-ui/core/styles";
import {ButtonField, SelectField, useModelContext} from "@rx-suite/ui";
import {OrderModel, OrderStore, RequiredProcessesToUpdate} from "../../../../../stores/OrderStore";
import {CapabilityStore} from "../../../../../stores/machine/CapabilityStore";
import {currentUserCanPermission, SystemSettingModel, SystemSettingStore} from "@rx-suite/stores";
import {useIsSubmittingContext} from "../../../../../hooks/useIsSubmitting";
import {useSnackbar} from "notistack";
import {JobSide} from "../../../../../stores/RequiredProcessStore";
import { green, red } from "@material-ui/core/colors";
import {JobProcessStore} from "../../../../../stores/machine/JobProcessStore";
import {useRequiredProcessesContext} from "../../../../../hooks/useRequiredProcesses";

type ChangeProcessesDialogProps = {
    open: boolean;
    handleClose: () => void;
};

export type selectedRequiredProcesses = {
    [id: string]: {
        name: string | null;
        rank: number;
        left: {
            capabilities: Array<number>;
        } | null;
        right: {
            capabilities: Array<number>;
        } | null;
    };
};

const useStyles = makeStyles((theme: Theme) =>
    createStyles({
        addBtn: {
            backgroundColor: green[400],
            color: 'white'
        },
        removeBtn: {
            backgroundColor: red[400],
            color: 'white'
        }
    })
);

export const ChangeProcessesDialog: React.FunctionComponent<ChangeProcessesDialogProps> = observer(({open, handleClose}) => {

    const classes = useStyles();

    const model = useModelContext() as OrderModel;
    const {enqueueSnackbar} = useSnackbar();

    const [loading, setLoading] = React.useState<boolean>(false);

    const {
        isSubmitting,
        setIsSubmitting
    } = useIsSubmittingContext();

    const { requiredProcesses, refreshRequiredProcesses } = useRequiredProcessesContext();

    const [selectedRequiredProcesses, setSelectedRequiredProcesses] = React.useState<selectedRequiredProcesses>({});

    const systemSettingModel = SystemSettingStore.selectedModel as SystemSettingModel;
    const displayRightLensFirst = systemSettingModel?.displayRightLensFirst();

    const currentRequiredProcesses = React.useMemo(() => {

        const allJobProcesses: selectedRequiredProcesses = {};
        JobProcessStore.models.forEach((jobProcess) => {
            if (jobProcess.is_control_process) {
                return; // do not add control processes
            }
            allJobProcesses[jobProcess.getId()] = {
                name: jobProcess.resolvedName,
                rank: jobProcess.rank,
                left: null,
                right: null
            };
        });

        requiredProcesses.forEach((requiredProcess) => {
            allJobProcesses[requiredProcess.job_process_id.toString()][requiredProcess.side] = {
                capabilities: requiredProcess.capability_ids
            }
        });

        setSelectedRequiredProcesses(JSON.parse(JSON.stringify(allJobProcesses)));

        return allJobProcesses;

    }, [requiredProcesses]);

    const onClose = React.useCallback((event, reason) => {

        if (reason === 'backdropClick' || reason === 'escapeKeyDown') {
            return; // disable these ways to close dialog
        }

        if (loading) {
            return; // can't close the dialog while submitting
        }

        setSelectedRequiredProcesses(JSON.parse(JSON.stringify(currentRequiredProcesses)));
        handleClose();

    }, [loading, currentRequiredProcesses, handleClose]);

    const handleCheckAll = React.useCallback((side: 'left' | 'right', isChecked: boolean) => {

        setSelectedRequiredProcesses((prevState) => {

            const updatedState: selectedRequiredProcesses = {}

            for (const jobProcessId in prevState) {
                const jobProcessInfo = prevState[jobProcessId];

                updatedState[jobProcessId] = {
                    ...jobProcessInfo,
                    [side]: isChecked ? {
                        capabilities: []
                    } : null
                }
            }

            return updatedState;
        });
    }, [selectedRequiredProcesses])

    const handleCheck = React.useCallback((jobProcessId: string, side: 'left' | 'right', isChecked: boolean) => {

        setSelectedRequiredProcesses((prevState) => ({
            ...prevState,
            [jobProcessId]: {
                ...prevState[jobProcessId],
                [side]: isChecked ? {
                    capabilities: []
                } : null
            }
        }));

    }, [selectedRequiredProcesses]);

    const handleChangeCapability = React.useCallback((jobProcessId: string, side: 'left' | 'right', newCapabilities: Array<number>) => {

        setSelectedRequiredProcesses((prevState) => ({
            ...prevState,
            [jobProcessId]: {
                ...prevState[jobProcessId],
                [side]: {
                    capabilities: newCapabilities
                }
            }
        }));

    }, [selectedRequiredProcesses]);

    const capabilityOptions = React.useMemo(() => {
        const options: {
            [k: string]: Array<{
                label: string;
                value: number;
            }>;
        } = {};

        CapabilityStore.models.forEach((c) => {
            if (!options[c.job_process_id.toString()]) {
                options[c.job_process_id.toString()] = [];
            }
            options[c.job_process_id.toString()].push({
                label: c.resolvedName,
                value: c.id
            });
        });

        return options;
    }, [CapabilityStore.models]);

    const dialogContent = React.useMemo(() => {

        const requiredProcessesArr = [];
        for (const [key, value] of Object.entries(selectedRequiredProcesses)) {
            requiredProcessesArr.push({
                job_process_id: key,
                ...value
            });
        }

        const sortedRequiredProcessesArr = requiredProcessesArr.sort((a, b) => {
            return a.rank - b.rank;
        });

        const sides: Array<JobSide> = displayRightLensFirst ? ['right', 'left'] : ['left', 'right'];

        return <Box>
            <Grid container>
                <Grid container item>
                    <Grid item xs={4}>
                        <Box display={'flex'} alignItems={'flex-end'}>
                            <Typography variant={'h6'}>{t`Process Name`}</Typography>
                        </Box>
                    </Grid>
                    {sides.map((side: JobSide) => {
                        const sideText = {
                            left: t`Left`,
                            right: t`Right`
                        }[side];

                        return <Grid item xs={4}>
                            <Box display={'flex'} alignItems={'flex-end'}>
                                <Typography variant={'h6'}>{sideText}</Typography>
                            </Box>
                        </Grid>;
                    })}
                </Grid>
                <Grid item xs={12} >
                    <Box mb={1}>
                        <Divider/>
                    </Box>
                </Grid>
                <Grid item xs={4}>
                    {/* Empty */}
                </Grid>
                {sides.map((side: JobSide) => {
                    return <Grid item xs={4}>
                        <Box display={'flex'} alignItems={'flex-end'}>
                            <ButtonField
                                className={classes.addBtn}
                                text={t`Add All`}
                                size={'small'}
                                onClick={() => {handleCheckAll(side, true)}}
                            />
                            <ButtonField
                                className={classes.removeBtn}
                                text={t`Remove All`}
                                size={'small'}
                                onClick={() => {handleCheckAll(side, false)}}
                            />
                        </Box>
                    </Grid>;
                })}
                {sortedRequiredProcessesArr.map((requiredProcess) => {

                    const jobProcessId = requiredProcess.job_process_id;
                    const processOptions = capabilityOptions[jobProcessId.toString()] || [];

                    const createSideComponent = (side: JobSide) => {

                        const processSide = requiredProcess[side];

                        return <Grid item xs={4}>
                            <Box display={'flex'}>
                                <Checkbox
                                    checked={!!processSide}
                                    onChange={(event) => {
                                        handleCheck(jobProcessId, side, event.target.checked);
                                    }}
                                />
                                {(!!processSide && processOptions.length) ? <SelectField
                                    options={processOptions}
                                    onChange={(e, value) => {
                                        handleChangeCapability(jobProcessId, side, value);
                                    }}
                                    multiple={true}
                                    value={processOptions.filter((capability) => {
                                        if (!processSide) {
                                            return false;
                                        }
                                        return processSide.capabilities.includes(capability.value);
                                    }).map((p) => p.value)}
                                /> : null}
                            </Box>
                        </Grid>;
                    };

                    const leftComponent = createSideComponent('left');
                    const rightComponent = createSideComponent('right');

                    const componentsToShow = displayRightLensFirst
                        ? [rightComponent, leftComponent]
                        : [leftComponent, rightComponent]
                    ;

                    return <Grid container item>
                        <Grid item xs={4}>
                            <Typography variant={'body1'}>{requiredProcess.name}</Typography>
                        </Grid>
                        {componentsToShow.map((e) => (e))}
                    </Grid>
                })}
            </Grid>
        </Box>;

    }, [selectedRequiredProcesses]);

    const handleSubmit = React.useCallback(async (e) => {

        const canChangeProcesses = currentUserCanPermission({
            target: OrderStore.permissionTarget as string,
            action: 'ChangeProcesses'
        });

        if (!canChangeProcesses) {
            enqueueSnackbar('Missing permission to change processes', { variant: 'error' });
            return null;
        }

        setLoading(true);
        setIsSubmitting(true);

        const dataToUpdate: Array<RequiredProcessesToUpdate> = [];
        for (const [k, v] of Object.entries(selectedRequiredProcesses)) {
            dataToUpdate.push({
                job_process_id: Number(k),
                left: v.left,
                right: v.right
            })
        }
        const resp = await model.updateRequiredProcesses(dataToUpdate);

        refreshRequiredProcesses(resp.required_process_ids);

        setIsSubmitting(false);
        setLoading(false);
        handleClose();

    }, [model, selectedRequiredProcesses, handleClose]);

    return (
        <Dialog open={open} onClose={onClose} maxWidth={'md'} fullWidth>
            <DialogTitle>{t`Change Job Processes`}</DialogTitle>
            <DialogContent>
                <Box p={2}>
                    {dialogContent}
                </Box>
                <DialogActions>
                    <ButtonField
                        text={t`Close`}
                        onClick={onClose}
                        color={'secondary'}
                        variant={'text'}
                        disabled={loading}
                    />
                    <ButtonField
                        text={t`Submit`}
                        onClick={handleSubmit}
                        disabled={isSubmitting}
                        loading={loading}
                        color={'secondary'}
                        variant={'text'}
                    />
                </DialogActions>
            </DialogContent>
        </Dialog>
    );
});
