import React from "react";
import {observer} from "mobx-react-lite";
import {
    Dialog,
    DialogTitle,
    Box,
    DialogActions,
    DialogContent,
    Typography,
    Grid, Divider
} from "@material-ui/core";
import {t} from "@lingui/macro";
import {ButtonField, SelectField, useFlag, useModelContext} from "@rx-suite/ui";
import {OrderModel, OrderStore} from "../../../../../stores/OrderStore";
import {useIsSubmittingContext} from "../../../../../hooks/useIsSubmitting";
import {useSnackbar} from "notistack";
import {MachineModel, MachineStore} from "../../../../../stores/machine/MachineStore";
import {JobProcessModel, JobProcessStore} from "../../../../../stores/machine/JobProcessStore";
import { currentUserCanPermission } from "@rx-suite/stores";
import {useRequiredProcessesContext} from "../../../../../hooks/useRequiredProcesses";

export type ProcessMachine = {
    process: JobProcessModel;
    machine: MachineModel | null;
};

type OrderScheduleDialogProps = {
    open: boolean;
    handleClose: () => void;
};

type SelectedProcessMachine = {
    processId: string;
    processName: string;
    machineId: string | null;
};

export const OrderScheduleDialog: React.FunctionComponent<OrderScheduleDialogProps> = observer(({open, handleClose}) => {

    const model = useModelContext() as OrderModel;
    const {enqueueSnackbar} = useSnackbar();
    const { requiredProcesses } = useRequiredProcessesContext();

    const [loading, setLoadingTrue, setLoadingFalse, toggleLoading] = useFlag(false);
    const [selectedProcessMachines, setSelectedProcessMachines] = React.useState<Array<SelectedProcessMachine>>([]);

    const {
        isSubmitting,
        setIsSubmitting
    } = useIsSubmittingContext();

    const currentProcessMachines = React.useMemo(() => {

        const machines = model.machine_schedule_ids.map((orderMachineId) => {
            return MachineStore.models.find((machine) => machine.id === orderMachineId)
        }).filter(m => !!m) as MachineModel[];

        const processes: {
            [k: string]: ProcessMachine
        } = {};

        requiredProcesses.forEach((rProcess) => {

            if (rProcess.job_process_id in processes) {
                return;
            }

            const jobProcess = JobProcessStore.models.find((e) => e.id === rProcess.job_process_id);
            if (!jobProcess) {
                return;
            }

            processes[rProcess.job_process_id] = {
                process: jobProcess,
                machine: machines.find((machine) => machine.job_process_id === jobProcess.id) || null
            };
        });

        const sortedProcessMachines = Object.values(processes).sort((a, b) => {
            const aVal = a.process.rank as number;
            const bVal = b.process.rank as number;
            return aVal - bVal;
        });

        const currProcessMachines = sortedProcessMachines.map((cProcessMachine) => {
            return {
                processId: cProcessMachine.process.id as string,
                processName: cProcessMachine.process.resolvedName,
                machineId: cProcessMachine.machine ? (cProcessMachine.machine.id as string) : null
            };
        });

        setSelectedProcessMachines(JSON.parse(JSON.stringify(currProcessMachines)));

        return currProcessMachines;

    }, [model.machine_schedule_ids, requiredProcesses]);

    const onClose = React.useCallback((event, reason) => {

        if (reason === 'backdropClick' || reason === 'escapeKeyDown') {
            return; // disable these ways to close dialog
        }

        if (loading) {
            return; // can't close the dialog while submitting
        }

        setSelectedProcessMachines(JSON.parse(JSON.stringify(currentProcessMachines)));

        handleClose();

    }, [loading, currentProcessMachines, handleClose]);

    const machineOptionsForProcess = React.useCallback((process_id: string) => {

        const machines = MachineStore.models.filter((machine) => machine.job_process_id === Number(process_id));

        return machines.map((machine) => {
            return {
                label: machine.name,
                value: machine.id
            };
        });

    }, [MachineStore.models]);

    const dialogContent = React.useMemo(() => {

        return <Grid container>
            {selectedProcessMachines.map((processMachine, i) => {
                const processId = processMachine.processId;
                const processName = processMachine.processName;
                const machineId = processMachine.machineId;

                return <Grid container item alignItems={'center'}>
                    {i !== 0 && <Grid xs={12}>
                        <Box my={2}/>
                        <Divider variant={'middle'}/>
                    </Grid>}
                    <Grid xs={3}>
                        <Typography>{processName}</Typography>
                    </Grid>
                    <Grid xs={9}>
                        <SelectField
                            label={t`Schedule to Machine`}
                            onChange={(name, value) => {
                                setSelectedProcessMachines(prevState => {
                                    const newState = prevState.slice();
                                    newState[i].machineId = value;
                                    return newState;
                                });
                            }}
                            value={machineId}
                            options={machineOptionsForProcess(processId)}
                            disabled={isSubmitting || loading}
                            clearable
                        />
                    </Grid>
                </Grid>
            })}
        </Grid>;

    }, [selectedProcessMachines, machineOptionsForProcess, isSubmitting, loading]);

    const handleSubmit = React.useCallback(async (e) => {

        const canChangeProcesses = currentUserCanPermission({
            target: OrderStore.permissionTarget as string,
            action: 'MachineSchedule'
        });

        if (!canChangeProcesses) {
            enqueueSnackbar('Missing permission to change processes', { variant: 'error' });
            return null;
        }

        setLoadingTrue();
        setIsSubmitting(true);

        const machineIds = selectedProcessMachines.map((sProcessMachine) => sProcessMachine.machineId).filter(mId => !!mId) as string[];

        try {
            await model.setOrderMachineSchedule(machineIds);
        } catch (e) {
            console.error(e);
            enqueueSnackbar(t`Failed to set order machine schedule`, { variant: 'error'});
        }

        setIsSubmitting(false);
        setLoadingFalse();
        handleClose();

    }, [selectedProcessMachines, model, handleClose]);

    return (
        <Dialog open={open} onClose={onClose} maxWidth={'sm'} fullWidth>
            <DialogTitle>{t`Process Scheduling`}</DialogTitle>
            <DialogContent>
                <Box p={2}>
                    {dialogContent}
                </Box>
                <DialogActions>
                    <ButtonField
                        text={t`Close`}
                        onClick={onClose}
                        color={'secondary'}
                        variant={'text'}
                        disabled={loading}
                    />
                    <ButtonField
                        text={t`Submit`}
                        onClick={handleSubmit}
                        disabled={isSubmitting}
                        loading={loading}
                        color={'secondary'}
                        variant={'text'}
                    />
                </DialogActions>
            </DialogContent>
        </Dialog>
    );
});
