import React from "react";
import {observer} from "mobx-react-lite";
import {Box, createStyles, darken, Grid, Theme} from "@material-ui/core";
import {makeStyles} from "@material-ui/core/styles";
import classNames from "classnames";
import {grey} from "@material-ui/core/colors";
import {CellSize, OrderProcessesTableCell} from "./OrderProcessesTableCell";
import {ProcessNameCell} from "./cell_components/ProcessNameCell";
import { ActionsCell } from "./cell_components/ActionsCell";
import {CapabilitiesCell} from "./cell_components/CapabilitiesCell";
import {StateCell} from "./cell_components/StateCell";
import {OrderModel} from "../../../../stores/OrderStore";
import {SystemSettingModel, SystemSettingStore} from "@rx-suite/stores";
import {RequiredProcessGroup} from "../OrderProcessesTable";
import {RequiredProcessModel, RequiredProcessState} from "../../../../stores/RequiredProcessStore";

type OrderProcessesTableRowProps = {
    order: OrderModel;
    rowData: RequiredProcessGroup;
    isLastRow: boolean;
};

const useStyles = makeStyles((theme: Theme) =>
    createStyles({
        gridRow: {
            borderStyle: 'solid',
            borderColor: grey[400],
            borderLeftWidth: '1px',
            borderTopWidth: '1px',
            borderBottomWidth: '0',
            borderRightWidth: '1px'
        },
        lastRowOfGrid: {
            borderBottomWidth: '1px'
        },
        noData: {
            display: 'flex',
            alignItems: 'center',
            justifyContent: 'center',
            height: '100%',
            backgroundColor: darken(grey[300], 0.1)
        }
    })
);

export const OrderProcessesTableRow: React.FunctionComponent<OrderProcessesTableRowProps> = observer(({order, rowData, isLastRow}) => {

    const classes = useStyles();

    const systemSettingModel = SystemSettingStore.selectedModel as SystemSettingModel;
    const displayRightLensFirst = systemSettingModel?.displayRightLensFirst();

    const colorFunc = React.useCallback((state: RequiredProcessState, isRework) => {

        if (isRework) {
            return {
                text: 'rgb(84,0,206)',
                background: 'rgba(100, 0, 255, 0.2)'
            }
        }

        return {
            "NOTSTARTED": {
                text: 'rgb(80,80,80)',
                background: 'rgba(240, 240, 240, 0.4)'
            },
            "STARTED": {
                text: 'rgb(196,123,22)',
                background: 'rgba(255, 225, 35, 0.4)'
            },
            "FINISHED": {
                text: 'rgb(40,121,17)',
                background: 'rgba(60, 183, 29, 0.4)'
            },
            "ABORTED": {
                text: 'rgb(154,38,13)',
                background: 'rgba(239, 55, 19, 0.4)'
            }
        }[state];

    }, []);

    const sortedCellComponents = React.useMemo(() => {

        const leftData = rowData.left;
        const rightData = rowData.right;

        if (!leftData && !rightData) {
            throw 'No left or right required process';
        }

        const completeAtInfo = order.scheduled_processes.find((e) => e.name === rowData.name);

        const initialData: Array<JSX.Element> = [
            <ProcessNameCell
                processName={rowData.display_name}
                completeAt={!completeAtInfo ? null : completeAtInfo.complete_at}
            />
        ];

        const sidedCells = (sideData: RequiredProcessModel | null) => {

            if (!sideData) {
                const noDataComponent = <Box className={classes.noData}></Box>;

                return [noDataComponent, noDataComponent]; // capabilities cell + state cell
            }

            return [
                <CapabilitiesCell
                    capability_ids={sideData.capability_ids}
                    colors={colorFunc(sideData.state, sideData.is_rework)}
                />,
                <StateCell
                    state={sideData.is_rework ? 'REWORK' : sideData.state}
                    colors={colorFunc(sideData.state, sideData.is_rework)}
                />
            ]
        };

        const leftCells = sidedCells(leftData);
        const rightCells = sidedCells(rightData);

        const disableRework = (): boolean => {
            if(leftData?.is_rework || rightData?.is_rework) {
                return true;
            }

            if(leftData?.state !== "FINISHED" && rightData?.state !== "FINISHED"){
                return true;
            }

            if(!leftData || !rightData){
                return true;
            }
            
            return false;
        }

        const actionData = [
            <ActionsCell
                order={order}
                jobProcessRank={rowData.rank}
                requiredProcessIds={{
                    left: !leftData ? undefined : (leftData.id as number),
                    right: !rightData ? undefined : (rightData.id as number)
                }}
                disableRework={disableRework()}
            />
        ];

        return displayRightLensFirst ? initialData.concat(rightCells, leftCells, actionData) : initialData.concat(leftCells, rightCells, actionData);

    }, [displayRightLensFirst, rowData]);

    const cellSizes: ReadonlyArray<CellSize> = React.useMemo(() => {
        return [3, 2, 1, 2, 1, 3];
    }, []);

    return <Grid container item xs={12}
                 className={classNames({
                     [classes.gridRow]: true,
                     [classes.lastRowOfGrid]: isLastRow
                 })}
    >
        {sortedCellComponents.map((cellComponent: any, i: number) => (
            <OrderProcessesTableCell
                key={`order-process-table-cell-${i}`}
                cellComponent={cellComponent}
                cellSize={cellSizes[i]}
                isLastCell={i === (sortedCellComponents.length - 1)}
            />
        ))}
    </Grid>;
});
