import React from "react";
import {observer} from "mobx-react-lite";
import {Box, createStyles, Dialog, DialogTitle, Divider, Grid, Theme, Typography} from "@material-ui/core";
import {makeStyles} from "@material-ui/core/styles";
import {RequiredProcessStateHistoryStore} from "../../../../../../stores/RequiredProcessStateHistoryStore";
import {BreadModel} from "@rx-suite/ui";
import {t} from "@lingui/macro";
import {SystemSettingModel, SystemSettingStore} from "@rx-suite/stores";
import {useRequiredProcessHistoryStateColumn} from "./useRequiredProcessHistoryStateColumn";
import {grey} from "@material-ui/core/colors";
import moment from "moment/moment";

type RequiredProcessHistoryDialogProps = {
    requiredProcessIds: {
        left?: number;
        right?: number;
    };
    open: boolean;
    handleClose: () => void;
};

const useStyles = makeStyles((theme: Theme) =>
    createStyles({
        noDataLayout: {
            display: 'flex',
            alignItems: 'flex-start',
            justifyContent: 'center',
            width: '100%',
            height: '100%'
        },
        noData: {
            display: 'flex',
            alignItems: 'center',
            justifyContent: 'center',
            width: '75%',
            height: '50%',
        },
        noDataText: {
            color: grey[400],
            fontSize: '2rem'
        }
    })
);

export const RequiredProcessHistoryDialog: React.FunctionComponent<RequiredProcessHistoryDialogProps> = observer(({requiredProcessIds, open, handleClose}) => {

    const classes = useStyles();

    const systemSettingModel = SystemSettingStore.selectedModel as SystemSettingModel;
    const displayRightLensFirst = systemSettingModel?.displayRightLensFirst();

    // Clone stores since we have side-by-side bread models, only clone once
    const storeClones = React.useMemo(() => {
        return {
            left: RequiredProcessStateHistoryStore.clone(),
            right: RequiredProcessStateHistoryStore.clone()
        };
    }, []);

    // Can't have this in useMemo because useRequiredProcessHistoryStateColumn has its own react hooks
    const columns = [
        useRequiredProcessHistoryStateColumn(),
        {
            Header: t`Source`,
            accessor: d => d.source,
            id: 'source',
            Cell: (cellInfo: {value: string }) => {
                return cellInfo.value;
            },
            sortable: false, // sort doesn't seem to work on cloned store
            filterable: false
        },
        {
            Header: t`Time`,
            accessor: d => d.time,
            id: 'time',
            Cell: (cellInfo: {value: string }) => {
                const systemSettings = SystemSettingStore.selectedModel as SystemSettingModel;
                return systemSettings.printDateTime(moment(cellInfo.value));
            },
            sortable: false, // sort doesn't seem to work on cloned store
            filterable: false
        }
    ];

    const dialogContent = React.useMemo(() => {

        const noDataComponent = <Box className={classes.noDataLayout}>
            <Box className={classes.noData}>
                <Typography className={classes.noDataText}>{t`Not Required`}</Typography>
            </Box>
        </Box>;

        const createTable = (label: string, side: 'left' | 'right') => {

            if (!storeClones) {
                return noDataComponent;
            }

            return <Box flex={1}>
                <Box display={'flex'} flexDirection={'row'} justifyContent={'center'}>
                    <Typography variant={'h6'}>{label}</Typography>
                </Box>
                <Box mb={2}>
                    <Divider variant={'middle'}/>
                </Box>
                {requiredProcessIds[side] ? <BreadModel
                        store={storeClones[side]}
                        Form={() => {}} // No form
                        columns={columns}
                        tableProps={{
                            showCreateButton: false,
                            defaultSorted: [{id: 'time', desc: true}],
                            defaultFiltered: [{id: 'required_process_id', value: requiredProcessIds[side], strictMatchFilter: true}],
                            suppressActionColumn: true,
                            suppressWrapper: true
                        }}
                    />
                    : noDataComponent
                }
            </Box>;
        };

        const leftTable = createTable(t`Left`, 'left');
        const spacing = <Box mx={1}/>;
        const rightTable = createTable(t`Right`, 'right');

        const tables = displayRightLensFirst ? [rightTable, spacing, leftTable] : [leftTable, spacing, rightTable];

        return <>
            {tables.map((e) => e)}
        </>;

    }, [storeClones, requiredProcessIds]);

    return (
        <Dialog open={open} onClose={handleClose} maxWidth={'lg'} fullWidth>
            <DialogTitle>{t`Required Process State History`}</DialogTitle>
            <Box p={2} display={'flex'}>
                {dialogContent}
            </Box>
        </Dialog>
    );
});
