import React from "react";
import {observer} from "mobx-react-lite";
import {Box, createStyles, IconButton, Theme, Typography} from "@material-ui/core";
import {useSnackbar} from "notistack";
import {makeStyles} from "@material-ui/core/styles";
import {red, grey} from "@material-ui/core/colors";
import {SettingStore} from "../../../../../../stores/SettingStore";
import {useIsSubmittingContext} from "../../../../../../hooks/useIsSubmitting";
import {OrderModel} from "../../../../../../stores/OrderStore";
import {t} from "@lingui/macro";
import {ButtonField} from "@rx-suite/ui";
import {SystemSettingModel, SystemSettingStore} from "@rx-suite/stores";
import {Close} from "@material-ui/icons";
import {JobSide} from "../../../../../../stores/RequiredProcessStore";
import {useRequiredProcessesContext} from "../../../../../../hooks/useRequiredProcesses";
import {JobProcessModel, JobProcessStore} from "../../../../../../stores/machine/JobProcessStore";

type RequiredProcessReworkSelectProps = {
    order: OrderModel;
    jobProcessRank: number;
    onReworkFinish: () => void;
};

const useStyles = makeStyles((theme: Theme) =>
    createStyles({
        reworkText: {
            fontWeight: 'bold',
            color: grey[600]
        },
        icon: {
            fontSize: '1.25rem',
            padding: theme.spacing(0.25)
        },
        errorIcon: {
            position: 'absolute',
            right: '36px',
            color: red[400]
        },
        sideBtn: {
            minWidth: '75px'
        }
    })
);

export const RequiredProcessReworkSelect: React.FunctionComponent<RequiredProcessReworkSelectProps> = observer(({order, jobProcessRank, onReworkFinish}) => {

    const classes = useStyles();

    const {enqueueSnackbar} = useSnackbar();

    const settings = SettingStore.selectedModel;

    const systemSettingModel = SystemSettingStore.selectedModel as SystemSettingModel;
    const displayRightLensFirst = systemSettingModel?.displayRightLensFirst();

    const {
        isSubmitting,
        setIsSubmitting
    } = useIsSubmittingContext();

    const sideArr: JobSide[] = React.useMemo(() => {
        return displayRightLensFirst ? ['right', 'left'] : ['left', 'right'];
    }, [displayRightLensFirst]);

    const getSideText = React.useCallback((side: JobSide) => {
        return {
            left: t`Left`,
            right: t`Right`
        }[side];
    }, []);

    const handleRework = React.useCallback(async (side: JobSide) => {
        setIsSubmitting(true);
        await order.reworkUpTo(jobProcessRank, side);
        setIsSubmitting(false);
        onReworkFinish();
    }, [jobProcessRank]);


    const disableSide = (side: JobSide): boolean => {
        const {requiredProcesses} = useRequiredProcessesContext()

        const oppositeSide = side === 'left' ? 'right' : 'left';

        const process = JobProcessStore.models.find(process => process.rank === jobProcessRank) as JobProcessModel;

        const oppositeProcess = requiredProcesses.find(reqProcess => reqProcess.job_process_id === process.id && reqProcess.side === oppositeSide);

        if(!oppositeProcess){
            return true;
        }

        return oppositeProcess.state !== 'FINISHED';
    }

    return (
        <Box ml={1} width={'100%'}>
            <Box display={'flex'} flexDirection={'column'} alignItems={'center'} justifyContent={'center'}>
                <Box>
                    <Typography
                        className={classes.reworkText}
                        variant={'body1'}
                    >
                        {t`Rework`}
                    </Typography>
                </Box>
                <Box>
                    <Box display={'flex'} flexDirection={'row'} justifyContent={'center'}>
                        {sideArr.map((side: JobSide) => (
                            <ButtonField
                                className={classes.sideBtn}
                                text={getSideText(side)}
                                onClick={async () => { await handleRework(side); }}
                                color={'primary'}
                                variant={'contained'}
                                size={'medium'}
                                loading={isSubmitting}
                                disabled={disableSide(side)}
                            />
                        ))}
                        <IconButton
                            className={classes.errorIcon}
                            onClick={onReworkFinish}
                            disabled={isSubmitting}
                        >
                            <Close/>
                        </IconButton>
                    </Box>
                </Box>
            </Box>
        </Box>
    );
});
