import React from "react";
import {observer} from "mobx-react-lite";
import {ButtonField, PaddedPaper, ScannerField} from "@rx-suite/ui";
import {useSnackbar} from "notistack";
import {routerStore} from "@rx-suite/data";
import { t } from "@lingui/macro";
import { Box } from "@material-ui/core";
import {findOrderByOrderNumber, OrderStore} from "../../../../stores/OrderStore";
import {Search} from "@material-ui/icons";

type OrderFindFieldProps = {};

export const OrderFindField: React.FunctionComponent<OrderFindFieldProps> = observer(({}) => {

    const {enqueueSnackbar} = useSnackbar();

    const [jobNumber, setJobNumber] = React.useState<string>('');

    const [fetching, setFetching] = React.useState<boolean>(false);

    const handleJobFinder = async () => {

        if (fetching) {
            enqueueSnackbar(`Already fetching another job`, {variant: "error"});
            return; // already fetching another request
        }

        if (jobNumber.trim() === '') {
            enqueueSnackbar(`${t`Cant search for a job without a number`}`, {variant: "error"});
            return;
        }

        setFetching(true);

        try {
            const modelData = await findOrderByOrderNumber(jobNumber);

            const model = OrderStore.createModel(modelData)
            OrderStore.setModel(model);
            enqueueSnackbar(`Job found: ${jobNumber}`, {variant: "success"});
            routerStore.history.push(`/job_workstation/jobs/${model.getId().toString()}/edit`);

        } catch (err) {
            const errKey = err?.response?.error;
            const errString = errKey === "JobDoesNotExist" ? `Job does not exist: ${jobNumber}` : `Error Loading Job: ${jobNumber}`;
            if (errKey !== "JobDoesNotExist") {
                console.error(errString);
            }
            enqueueSnackbar(errString, {variant: "error"});
        }

        setFetching(false);
        setJobNumber('');
    }

    const onChange = (e: any) => {
        setJobNumber(e.target.value);
    }

    const onEnterKey = async () => {
        if (fetching) {
            return; // already fetching another job
        }
        await handleJobFinder();
    }

    const onComplete = async (code: string) => {
        setJobNumber(code);
    }

    return (
        <PaddedPaper>
            <Box display={'flex'}>
                <Box flex={1}>
                    <ScannerField
                        inputConfig={{
                            fullWidth: true,
                            label: t`Job Number`,
                            value: jobNumber,
                            onChange: onChange,
                            onEnterKey: onEnterKey,
                            autoFocus: true
                        }}
                        scanConfig={{
                            onComplete: onComplete,
                        }}
                    />
                </Box>
                <ButtonField
                    text={'Search'}
                    startIcon={<Search/>}
                    onClick={handleJobFinder}
                    buttonType={'submit'}
                    color={'secondary'}
                    disabled={fetching}
                    loading={fetching}
                />
            </Box>
        </PaddedPaper>
    );
});