import * as React from 'react';
import {BreadModel, BreadModelListBadge, makeColumn, ModelInput, ModelSelect, SelectField} from "@rx-suite/ui";
import { t } from "@lingui/macro";
import {DeviceModel, DeviceStore} from "../../../stores/Device/DeviceStore";
import {observer} from "mobx-react-lite";
import {OpcuaDeviceForm} from "./form/OpcuaDeviceForm";
import {useHasControlActionColumn} from "../../../hooks/table_columns/useHasControlActionColumn";
import {Box, Grid, LinearProgress} from "@material-ui/core";
import {DeviceTypeStore} from "../../../stores/Device/DeviceTypeStore";
import {Column, Filter, ReactTableFunction} from "react-table";

const DevicesBulkFormInput: React.FunctionComponent = () => {
    const deviceTypeSelectOptions = React.useMemo(() => {
        return DeviceTypeStore.models.map(type => ({label: type.name, value: type.getId()}));
    }, [DeviceTypeStore.models]);

    return <React.Fragment>
        <Box ml={1} flex={1}>
            <ModelSelect
                name={'device_type_id'}
                options={deviceTypeSelectOptions}
                clearable
                loading={DeviceTypeStore.isLoading}
            />
        </Box>
        <Box flex={1}>
            <ModelInput name={'name'}/>
        </Box>
    </React.Fragment>
};

export const OpcuaDevicesBreadModel = observer(() => {
    DeviceTypeStore.getAll();

    const columns = React.useMemo(() => {
        const deviceTypeSelectOptions = () => {
            return DeviceTypeStore.models
                .map(type => ({label: type.name, value: type.getId()}))
                .concat({label: 'All', value: ""});
        }

        return [
            makeColumn('name', t`Name`, 'string'),
            {
                Header: t`Device Type`,
                accessor: (device: DeviceModel) => device,
                id: 'device_type_id',
                Cell: (cellInfo: {value: DeviceModel }) => {
                    const type = DeviceTypeStore.models.find(type => cellInfo.value.device_type_id === type.id)

                    if(!type){
                        return null;
                    }

                    return type.name;
                },
                filterable: true,
                Filter: ({filter, onChange}: { filter: Filter, onChange: ReactTableFunction }) => (
                    <SelectField
                        options={deviceTypeSelectOptions()}
                        onChange={(name, value) => onChange(value)}
                        value={filter ? filter.value : ""}
                    />
                ),
                sortable: false,
                strictMatchFilter: true
            },
            {
                Header: t`Scanners`,
                accessor: (device: DeviceModel) => device,
                maxWidth: 135,
                id: 'input_with_scanners',
                Cell: (cellInfo: {value: DeviceModel }) => {
                    if(cellInfo.value.inputs_with_scanners.length === 0){
                        return null;
                    }

                    return (
                        <Grid container direction={'column'}>
                            {
                                cellInfo.value.inputs_with_scanners.map(scannerName => (
                                    <Box>
                                        <BreadModelListBadge text={scannerName}/>
                                    </Box>

                                ))
                            }
                        </Grid>
                    );
                },
                sortable: false
            },
            useHasControlActionColumn()
        ]
    }, [DeviceTypeStore.models]);

    const bulkCreateColumns: Column[] = React.useMemo(() => {
        return [
            {
                Header: t`Name`,
                accessor: 'name',
                id: 'name',
                Cell: (cellInfo: { value: string }) => {
                    return <Box display={'flex'} alignItems={'center'} height={'100%'}>
                        {cellInfo.value}
                    </Box>;
                }
            },
            {
                Header: t`Type`,
                accessor: (device: DeviceModel) => device,
                id: '_device_type_id',
                Cell: (cellInfo: { value: DeviceModel }) => {
                    const deviceType = DeviceTypeStore.models.find((type) => type.id === cellInfo.value.device_type_id);

                    if (!deviceType) {
                        return null;
                    }

                    return <Box display={'flex'} alignItems={'center'} height={'100%'}>
                        {deviceType.name}
                    </Box>;
                }
            }
        ]
    }, [DeviceTypeStore.models]);

    if(DeviceTypeStore.isLoading){
        return <LinearProgress/>;
    }

    return (
        <BreadModel
            store={DeviceStore}
            Form={OpcuaDeviceForm}
            columns={columns}
            bulkCreateProps={
                {
                    bulkCreateInputs: <DevicesBulkFormInput/>,
                    tableColumns: bulkCreateColumns
                }
            }
        />
    )
});