import React from "react";
import {FormProps, ModelInput, ModelProvider, ModelSelect, PaddedPaper} from "@rx-suite/ui";
import {observer} from "mobx-react-lite";
import {t} from "@lingui/macro";
import {DeviceTypeModel, DeviceTypeStore} from "../../../../stores/Device/DeviceTypeStore";
import {DeviceModel} from "../../../../stores/Device/DeviceStore";
import {OpcuaConnectorsTab} from "./tabs/connectors/OpcuaConnectorsTab";
import {OpcuaConstantsTab} from "./tabs/timers_constants/OpcuaConstantsTab";
import {OpcuaScannersTab} from "./tabs/scanners/OpcuaScannersTab";
import {OpcuaRoutingCommandTab} from "./tabs/routing_commands/OpcuaRoutingCommandTab";
import {TabsContainer} from "../../../../components/generic/TabsContainer";
import {DeviceDataTrackingTab} from "../../../../components/devices/form/DeviceDataTrackingTab";
import {useDeviceConnectors} from "../../../../hooks/device_form/useDeviceConnectors";
import {Box, LinearProgress} from "@material-ui/core";
import { OpcuaExtraRoutingRulesTab } from "./tabs/extra_routing_rules_tab/OpcuaExtraRoutingRulesTab";
import {useRouterOnline} from "../../../../hooks/useRouterOnline";
import {OpcuaDeviceResetButton} from "../../../../components/devices/form/OpcuaDeviceResetButton";
import {RouterResourceModelForm} from "../../../../components/router/RouterResourceModelForm";

type OpcuaDeviceFormProps = {
    model: DeviceModel
} & Omit<FormProps, 'model'>

export const OpcuaDeviceForm: React.FunctionComponent<OpcuaDeviceFormProps> = observer(({model}) => {
    DeviceTypeStore.getAll();

    React.useEffect(() => {
        useDeviceConnectors(model);
    }, [model]);

    const deviceTypeOptions = React.useMemo(() => {
        return DeviceTypeStore.models.map((deviceType: DeviceTypeModel) => {
            return {
                label: deviceType.name,
                value: deviceType.getId()
            }
        })
    }, [DeviceTypeStore.models]);

    const routingRetryDelayHint = {
        title: t`Routing Retry Delay`,
        text: t`Delay before another routing attempt is made when a tray cannot move.`
    }

    const delayOnCommandCompleteHint = {
        title: t`Delay on Command Complete`,
        text: t`Delay before another routing attempt is made when an ongoing command is completed.`
    }

    if(DeviceTypeStore.isLoading){
        return <LinearProgress/>
    }

    return(
        <React.Fragment>
            <PaddedPaper>
                <RouterResourceModelForm model={model} suppressImportExport suppressMakeCopy suppressWrapper postFormActions={<OpcuaDeviceResetButton/>}>
                    <ModelSelect
                        name={'device_type_id'}
                        label={t`Device type`}
                        options={deviceTypeOptions}
                        disabled={!model.isNew}
                    />
                    <ModelInput name={'name'} disabled={model.isNew ? false : useRouterOnline()}/>
                    <ModelInput
                        name={'routing_retry_delay'}
                        hint={routingRetryDelayHint}
                    />
                    <ModelInput
                        name={'delay_on_command_complete'}
                        hint={delayOnCommandCompleteHint}
                    />
                </RouterResourceModelForm>
            </PaddedPaper>
            {
                !model.isNew &&
                <Box mt={1}>
                    <ModelProvider value={model}>
                        <PaddedPaper>
                            <TabsContainer tabs={
                                [
                                    {
                                        label: t`Connectors`,
                                        component: <OpcuaConnectorsTab/>,
                                        defaultTab: true
                                    },
                                    {
                                        label: t`Scanners`,
                                        component: <OpcuaScannersTab/>,
                                    },
                                    {
                                        label: t`Timers & Constants`,
                                        component: <OpcuaConstantsTab/>
                                    },
                                    {
                                        label: t`Routing Commands`,
                                        component: <OpcuaRoutingCommandTab/>
                                    },
                                    {
                                        label: t`Extra Routing Rules`,
                                        component: <OpcuaExtraRoutingRulesTab/>
                                    },
                                    {
                                        label: t`Data Tracking`,
                                        component: <DeviceDataTrackingTab/>,
                                    }
                                ]
                            }/>
                        </PaddedPaper>
                    </ModelProvider>
                </Box>
            }
        </React.Fragment>


    )
});