import * as React from "react";
import {observer} from "mobx-react-lite";
import {FormProps, ModelInput, ModelSelect, SelectField} from "@rx-suite/ui";
import {ConnectorModel} from "../../../../../../stores/ConnectorStore";
import {ScannerStore} from "../../../../../../stores/ScannerStore";
import {t} from "@lingui/macro";
import {SettingStore} from "../../../../../../stores/SettingStore";
import {ModelFormDialog} from "../../../../../../components/generic/ModelFormDialog";
import {useRouterOnline} from "../../../../../../hooks/useRouterOnline";

type OpcuaScannerDialogFormProps = {
    model: ConnectorModel;
    setModel: React.Dispatch<React.SetStateAction<ConnectorModel | null>>
} & Omit<FormProps, 'model'>

type ScannerSource = 'opcua' | 'rx-scanners' | 'none';

export const OpcuaScannerDialogForm: React.FunctionComponent<OpcuaScannerDialogFormProps> = observer(({model, setModel}) => {
    const settings = SettingStore.selectedModel;

    React.useEffect(() => {
        if(!settings || !settings.use_rx_scanners){
            return;
        }
        ScannerStore.getAll();
    }, [settings])

    const scannerSourceFromModel = React.useMemo((): ScannerSource=> {
        if(model.viewModel.opcua_scanner_reference !== null){
            return 'opcua'
        }else if(model.viewModel.rx_scanner_reference !== null){
            return 'rx-scanners'
        }
        return 'none';
    }, [model]);

    const [scannerSource, setScannerSource] = React.useState<string>(scannerSourceFromModel);

    const handleChangeScannerTarget = (name: string | undefined, value: string) => {
        const defaultOpcuaVar = (): string => {
            let defaultName = 'SC';

            if(model.name == 'trunk-in'){
                defaultName += 'T'
            }else if(model.name == 'feed-in'){
                defaultName += 'F'
            }
            else{
                defaultName = '';
            }

            return defaultName;
        }

        model.nestedAssignment('scanner_retry', null);

        if(value === 'opcua'){
            model.nestedAssignment('opcua_scanner_reference', defaultOpcuaVar());
        }else{
            model.nestedAssignment('opcua_scanner_reference', null);
        }

        if(value !== 'rx-scanners'){
            model.nestedAssignment('rx_scanner_reference', null);
        }

        if(value !== 'none'){
            model.nestedAssignment('scanner_retry', 3);
        }

        setScannerSource(value);
    }

    const scannerOptions = () => {
        let scanners = ScannerStore.models.map((scanner) => {return {label: scanner.name, value: scanner.id}});
        scanners.push({label: 'none', value: null});
        return scanners;
    }

    const scannerTargets = () => {
        const choices =  [
            {label: t`None`, value: 'none'},
            {label: t`OPCUA Node`, value: 'opcua'}
        ]

        if(settings?.use_rx_scanners){
            choices.push({label: t`Rx-Scanners`, value: 'rx-scanners'})
        }

        return choices;
    }

    return (
        <ModelFormDialog
            title={t`Edit Scanner`}
            model={model}
            setModel={setModel}
        >
            <SelectField
                label={t`Scanner Target`}
                onChange={handleChangeScannerTarget}
                value={scannerSource}
                options={scannerTargets()}
                disabled={useRouterOnline()}
            />
            {
                scannerSource === 'opcua' &&
                <ModelInput
                    label={t`OPCUA Variable`}
                    name={'opcua_scanner_reference'}
                    hint={
                        {
                            title: t`OPCUA Scanner Variable`,
                            text: t`This variable will be used to automatically calculate the id of the status and value fields. Ex: SCT will automatically generate SCTS (status field) and SCTV (value field).`
                        }
                    }
                    disabled={useRouterOnline()}
                />

            }
            {
                scannerSource === 'rx-scanners' &&
                <ModelSelect label={t`Scanner Name`}
                             name={t`rx_scanner_reference`}
                             options={scannerOptions()}
                             disabled={useRouterOnline()}
                />
            }
            {
                scannerSource !== 'none' &&
                <ModelInput name={'scanner_retry'}/>
            }
        </ModelFormDialog>
    )
})