import React from "react";
import {ButtonField, PaddedPaper, ScannerField, useFlag} from "@rx-suite/ui";
import {observer} from "mobx-react-lite";
import {Box, Grid, makeStyles, Typography} from "@material-ui/core";
import {AddCircle} from "@material-ui/icons";
import {t} from "@lingui/macro";
import {useSnackbar} from "notistack";
import {grey} from "@material-ui/core/colors";
import {createOrder, findOrderByOrderNumber, OrderStore} from "../../../../stores/OrderStore";
import {OrderInfo} from "../../../../components/orders/OrderInfo";
import {WarningDialog} from "../../../../components/generic/WarningDialog";

const useStyles = makeStyles((theme) => ({
    searchBox: {
        width: '90%',
        display: 'inline'
    },
    text: {
        color: grey[700]
    },
    icon: {
        color: grey[700],
        marginRight: theme.spacing(1)
    }
}));

export const OrderCreateWorkstation: React.FunctionComponent = observer(() => {
    const classes = useStyles();
    const {enqueueSnackbar} = useSnackbar();

    const [displayWarning, setDisplayWarningTrue, setDisplayWarningFalse, toggleDisplayWarning] = useFlag(false);
    const [lastCreatedId, setLastCreatedId] = React.useState<string | null>(null);
    const [jobNumber, setJobNumber] = React.useState<string>('');
    const [isFetching, setIsFetching] = React.useState(false);

    const lastCreatedModel = OrderStore.get(lastCreatedId);

    const checkOrderExists = React.useCallback(async (jobNumber: string) => {
        try {
            await findOrderByOrderNumber(jobNumber);
            return true;
        } catch (err) {
            return false;
        }
    }, []);

    const handleCreateJob = React.useCallback(async (validateOrderExists: boolean) => {

        if (jobNumber.trim().length <= 0) {
            enqueueSnackbar(t`Cannot search for a job without a number`, {variant: "error"});
            return;
        }

        setLastCreatedId(null);
        setIsFetching(true);

        try {
            if (validateOrderExists) {
                const orderExists = await checkOrderExists(jobNumber);

                if (orderExists) {
                    setDisplayWarningTrue();
                    setIsFetching(false);
                    return;
                }
            }

            const resp = await createOrder(jobNumber);

            const model = OrderStore.createModel(resp)
            OrderStore.setModel(model);
            setLastCreatedId(model.getId());

            setJobNumber('');
            enqueueSnackbar(`${t`Job Created`}`, {variant: "success"});

        } catch (err) {
            console.error(err);
            enqueueSnackbar(t`Error while creating job`, {variant: "error"});
        }

        setIsFetching(false);

    }, [jobNumber]);

    const onChange = React.useCallback((e: any) => {
        setJobNumber(e.target.value);
    }, []);

    const onComplete = React.useCallback(async (code: string) => {
        setJobNumber(code);
    }, []);

    const onEnterKey = React.useCallback(async () => {
        if (isFetching) { return; }
        await handleCreateJob(true);
    }, [isFetching, handleCreateJob]);

    return <Box>
        <Box mb={2}>
            <PaddedPaper>
                <Grid container alignItems={'center'}>
                    <AddCircle className={classes.icon}/>
                    <Typography variant={'h5'} className={classes.text}>{t`Create Job For Routing`}</Typography>
                </Grid>
                <Grid container justifyContent="space-evenly">
                    <Grid className={classes.searchBox} item>
                        <ScannerField
                            inputConfig={{
                                label: t`Job Number`,
                                value: jobNumber,
                                onChange: onChange,
                                onEnterKey: onEnterKey,
                                autoFocus: true
                            }}
                            scanConfig={{
                                onComplete: onComplete
                            }}
                        />
                    </Grid>
                    <ButtonField
                        text={t`Create`}
                        buttonType='submit'
                        color='secondary'
                        variant='contained'
                        loading={isFetching}
                        disabled={jobNumber.trim().length <= 0}
                        onClick={async () => { await handleCreateJob(true); }}
                    />
                </Grid>
            </PaddedPaper>
        </Box>
        {!!lastCreatedModel && <PaddedPaper>
            <OrderInfo model={lastCreatedModel}/>
        </PaddedPaper>}
        <WarningDialog
            warningMessage={t`This job number is already in use. Would you like to overwrite the job?`}
            open={displayWarning}
            onConfirm={async () => {
                setDisplayWarningFalse();
                await handleCreateJob(false);
            }}
            handleClose={setDisplayWarningFalse}
            critical
        />
    </Box>;
});