import {attribute, DataStore, Model, model} from '@rx-suite/data';
import * as yup from 'yup';
import { t } from "@lingui/macro"
import {action, computed, override} from "mobx";
import React from "react";
import {SettingStore} from "./SettingStore";
import {number} from "yup";

@model({
    transportLayer: "/api/connectors",
    permissionTarget: "Connector",
    routingInfo: {
        update: '/connectors/:id/edit'
    }
})
export class ConnectorModel extends Model {

    @attribute({
        label: t`Name`,
        validation: yup.string().required()
    })
    name: string = '';

    // parent component
    @attribute()
    component_id: number | null = null;

    @attribute()
    connector_type: 'input' | 'output' | 'other' = 'input';

    @attribute()
    control_action_config: ControlActionConfig | null = null;

    @attribute({
        label: t`Path Length`,
        validation: yup.number().min(1)
    })
    path_length: number = 1;

    @attribute({
        label: t`State`
    })
    state: 'online' | 'offline' | 'standby' = 'offline';

    @computed
    get isInput(): boolean {
        return this.connector_type === 'input';
    }

    @computed
    get hasScanner(): boolean {
        return this.viewModel.rx_scanner_reference !== null || this.viewModel.opcua_scanner_reference !== null;
    }

    @computed
    get fetchEnabled(): boolean {
        if(!this.viewModel.control_action_config || !this.viewModel.control_action_config.fetch){
            return false;
        }
        return this.viewModel.control_action_config.fetch.enabled;
    }

    @computed
    get forceToProcessEnabled(): boolean {
        if(!this.viewModel.control_action_config || !this.viewModel.control_action_config.setProcesses){
            return false;
        }
        return this.viewModel.control_action_config.setProcesses.process !== null;
    }

    @computed
    get validateProcessesEnabled(): boolean {
        if(!this.viewModel.control_action_config || !this.viewModel.control_action_config.validateProcesses){
            return false;
        }
        return this.viewModel.control_action_config.validateProcesses.process !== null;
    }

    @computed
    get addControlProcessEnabled(): boolean {
        if(!this.viewModel.control_action_config || !this.viewModel.control_action_config.addControlProcess){
            return false;
        }
        return this.viewModel.control_action_config.addControlProcess.process !== null;
    }


    /* OPC UA ONLY settings */

    @computed
    get useOpcua(): boolean {
        const settings = SettingStore.selectedModel;

        if(!settings){
            return false;
        }

        return settings.plcIsOpcua;
    }

    @attribute()
    variable: string | null = null;

    @attribute({
        label: t`Priority (%)`,
        validation: number().when(['useOpcua', 'isInput'],{
            is: true,
            then: schema => schema.min(0).max(1).typeError(t`Priority is required`),
            otherwise: schema => schema.nullable()
        })
    })
    priority: number = 0;

    @attribute({
        label: t`Action delay (milliseconds)`,
        validation: number().when(['useOpcua', 'isInput'],{
            is: true,
            then: schema => schema.min(0).typeError(t`Action delay is required`),
            otherwise: schema => schema.nullable()
        })
    })
    action_delay: number = 250;

    @attribute({
        label: t`Use OPCUA Node`,
        validation: yup.string().nullable()
    })
    opcua_scanner_reference: string | null = null;

    @attribute({
        label: t`Rx-Scanner Reference`,
        validation: yup.number().nullable()
    })
    rx_scanner_reference: number | null = null;

    @attribute({
        label: t`Scanner Retry`,
        validation: number().when('hasScanner',{
            is: true,
            then: schema => schema.min(1).max(10).typeError(t`Scanner Retry is a required field.`),
            otherwise: schema => schema.nullable()
        })
    })
    scanner_retry: number | null = null;

    /* MOP ONLY settings */

    @attribute({
        label: t`Path`,
        validation: number().when(['useOpcua', 'isInput'],{
            is: false,
            then: schema => schema.min(0).typeError(t`Path is required`),
            otherwise: schema => schema.nullable()
        })
    })
    path: string | null = null;

    @override
    handleChange(e: React.ChangeEvent<HTMLInputElement | HTMLTextAreaElement>) {
        super.handleChange(e);
        if(!this.fetchEnabled){
            this.nestedAssignment('control_action_config.fetch.restart', false);
        }
    }

    @override
    handleSelectChange(name: string | undefined, value: (string | Array<string> | boolean | Array<boolean> | number | Array<number>)) {
        super.handleSelectChange(name, value);

       if(name === 'control_action_config.setProcesses.process'){
            if(this.forceToProcessEnabled){
                this.nestedAssignment('control_action_config.setProcesses.restartNext', false);
                this.nestedAssignment('control_action_config.setProcesses.finishPrevious', false);
                this.nestedAssignment('control_action_config.setProcesses.setToState', "finished");
            }
        }
        if(name === 'control_action_config.validateProcesses.process'){
            if(this.validateProcessesEnabled){
                this.nestedAssignment('control_action_config.validateProcesses.ensureNoPriorAbort', false);
                this.nestedAssignment('control_action_config.validateProcesses.ensurePriorCompleted', false);
            }
        }
        if(name === 'control_action_config.addControlProcess.process'){
            if(this.addControlProcessEnabled){
                this.nestedAssignment('control_action_config.addControlProcess.processToInject', null);
                this.nestedAssignment('control_action_config.addControlProcess.filterProcess', null);
                this.nestedAssignment('control_action_config.addControlProcess.filterCapabilities', []);
            }
        }
        if(name === 'control_action_config.addControlProcess.filterProcess'){
            this.nestedAssignment('control_action_config.addControlProcess.filterCapabilities', []);
        }
    }

    @override
    updateFromJson(json: Model, isClone?: boolean) {
        super.updateFromJson(json, isClone);
        this.ensureControlActionConfig();
    }

    @override
    resetView() {
        // If the view is refreshed, we need to manually re-parse the control action JSON
        super.resetView();
        this.ensureControlActionConfig();
    }

    @action
    private ensureControlActionConfig(): void {
        const ensureSetting = (setting: string, defaultValue: any, subSettings: {name: string, value: any}[] = []): void => {
            if(!this.viewModel.control_action_config){
                return;
            }

            if(this.viewModel.control_action_config[setting] === undefined){
                this.viewModel.control_action_config[setting] = defaultValue;
            }

            for(let subSetting of subSettings){
                if(this.viewModel.control_action_config[setting][subSetting.name] === undefined){
                    this.viewModel.control_action_config[setting][subSetting.name] = subSetting.value;
                }
            }
        }

        if(!this.viewModel.isInput){
            return; // only inputs have control actions
        }

        if(this.viewModel.control_action_config === null){
            this.viewModel.control_action_config = {}
        }

        if(typeof this.viewModel.control_action_config === 'string'){
            this.viewModel.control_action_config = JSON.parse(this.viewModel.control_action_config) as ControlActionConfig;
        }

        ensureSetting('holdReject', false);
        ensureSetting('holdStarted', false);
        ensureSetting('splitJobsToOutput', false);
        ensureSetting('unreject', false);
        ensureSetting('forceFullPathResolve', false);

        ensureSetting('fetch', {}, [
            {name: 'enabled', value: false},
            {name: 'restart', value: false}
        ]);

        ensureSetting('setProcesses', {}, [
            {name: 'process', value: null},
            {name: 'setToState', value: "finished"},
            {name: 'finishPrevious', value: false},
            {name: 'restartNext', value: false},
        ]);

        ensureSetting('validateProcesses', {}, [
            {name: 'process', value: null},
            {name: 'ensureNoPriorAbort', value: false},
            {name: 'ensurePriorCompleted', value: false},
        ]);

        ensureSetting('addControlProcess', {}, [
            {name: 'process', value: null},
            {name: 'processToInject', value: null},
            {name: 'conditionalProcess', value: null},
        ]);
    }

    constructor() {
        super();
        super.setup();
    }
}

export type ControlActionConfig = {
    fetch?: {
        enabled: boolean;
        restart: boolean;
    }
    setProcesses?: {
        process: string | null;
        setToState: "finished" | "notstarted";
        finishPrevious: boolean;
        restartNext: boolean;
    }
    unreject?: boolean;
    holdReject?: boolean;
    holdStarted?: boolean;
    splitJobsToOutput?: boolean;
    forceFullPathResolve?: boolean;
    validateProcesses?: {
        process: string | null;
        ensureNoPriorAbort: boolean;
        ensurePriorCompleted: boolean;
    },
    addControlProcess?: {
        process: string | null;
        processToInject: string | null;
        filterProcess: string | null;
        filterCapabilities: string[];
    }
    [key: string]: any
}


export const ConnectorStore = ConnectorModel.store as DataStore<ConnectorModel>;
