import {attribute, DataStore, Model, model} from '@rx-suite/data';
import * as yup from 'yup';
import { t } from "@lingui/macro"
import {action, computed, override} from "mobx";
import {DeviceTypeStore} from "./DeviceTypeStore";
import {number, string} from "yup";
import {SettingStore} from "../SettingStore";
import React from "react";
import {useRouterOnline} from "../../hooks/useRouterOnline";

@model({
    transportLayer: '/api/devices',
    permissionTarget: "Device",
    routingInfo: {
        list: '/devices',
        create: '/devices/create',
        bulkCreate: '/devices/bulk_create',
        update: '/devices/:id/edit'
    }
})
export class DeviceModel extends Model {

    @attribute({
        label: t`Name`,
        validation: yup.string().required(t`Name is required`)
    })
    name: string = '';

    @attribute()
    component_id: number | null = null;

    @attribute()
    has_control_actions: boolean = false;

    @attribute({
        label: t`Send Job State when jobs arrive at this decision point`
    })
    send_job_state: boolean = false;

    @attribute({
        label: t`Send Job State as`,
        validation: string().when('send_job_state',{
            is: true,
            then: schema => schema.required().typeError(t`Send Job State as is required`),
            otherwise: schema => schema.nullable()
        })
    })
    send_job_state_as: string | null = null;

    @attribute({
        label: t`Job State Process`,
        validation: string().when('send_job_state',{
            is: true,
            then: schema => schema.required().typeError(t`Job State Process is required`),
            otherwise: schema => schema.nullable()
        })
    })
    job_state_process: string | null = null;

    @attribute({
        label: t`Always send job state for both lens side`
    })
    always_send_both_job_states: boolean = false;

    /* OPC UA ONLY settings */

    @computed
    get useOpcua(): boolean {
        const settings = SettingStore.selectedModel;

        if(!settings){
            return false;
        }

        return settings.plcIsOpcua;
    }

    @attribute({
        label: t`Device type`,
        validation: number().when('useOpcua',{
            is: true,
            then: schema => schema.required().typeError(t`Device Type is required`),
            otherwise: schema => schema.nullable()
        })
    })
    device_type_id: number | null = null;

    @attribute({
        label: t`Routing Retry Delay (seconds)`,
        validation: number().when('useOpcua',{
            is: true,
            then: schema => schema.required().min(0).typeError(t`Routing Retry Delay is required`),
            otherwise: schema => schema.nullable()
        })
    })
    routing_retry_delay: number | null = null;

    @attribute({
        label: t`Delay on Command Complete (milliseconds)`,
        validation: number().when('useOpcua',{
            is: true,
            then: schema => schema.required().min(0).typeError(t`Delay on Command Complete is required`),
            otherwise: schema => schema.nullable()
        })
    })
    delay_on_command_complete: number | null = null;

    @attribute()
    inputs_with_scanners: string[] = [];


    /* MOP ONLY settings */

    @attribute({
        label: t`Number of Outputs`,
        validation: number().when('useOpcua',{
            is: false,
            then: schema => schema.required().min(0).typeError(t`Number of Outputs is required`),
            otherwise: schema => schema.nullable()
        })
    })
    number_of_outputs: number | null = 2;


    @override
    handleSelectChange(name: string | undefined, value: (string | Array<string> | boolean | Array<boolean> | number | Array<number>)) {
        super.handleSelectChange(name, value);

        if(name === 'device_type_id'){
            const deviceType = DeviceTypeStore.models.find((deviceType) => deviceType.getId() === value);

            if(deviceType){
                this.nestedAssignment("name", deviceType.name);
                this.nestedAssignment("delay_on_command_complete", deviceType.delay_on_command_complete);
                this.nestedAssignment("routing_retry_delay", deviceType.routing_retry_delay);
            }
        }
    }

    @override
    handleChange(e: React.ChangeEvent<HTMLInputElement | HTMLTextAreaElement>) {
        super.handleChange(e);

        if(e.target.name === 'send_job_state'){
            if(this.viewModel.send_job_state === false){
                this.nestedAssignment('send_job_state_as', null);
                this.nestedAssignment('job_state_process', null);
                this.nestedAssignment('always_send_both_job_states', false);
            }
        }
    }

    @override
    updateFromJson(json: Model, isClone?: boolean) {
        super.updateFromJson(json, isClone);
        this.send_job_state = this.send_job_state_as !== null;
    }

    @action async requestPlcReset() {
        return await this.store.transportLayer.fetch(`/${this.getId()}/reset`, {
            method: "PUT",
            headers: {
                "Content-Type": "application/json"
            }
        });
    }

    canDelete(): boolean {
        return !useRouterOnline();
    }

    constructor() {
        super();
        super.setup();
    }
}

export const DeviceStore = DeviceModel.store as DataStore<DeviceModel>;
