import {consumer, DataStore, FetchParams, isArrayLike, Model} from "@rx-suite/data";
import {action, makeObservable, observable, runInAction} from "mobx";
import {Channel} from "actioncable";

type IncomingData = {
    action: string,
    model: Model,
    timestamp?: number
}

export class LatestModelsStore {
    @observable store: DataStore<Model>;
    @observable models: Model[];
    @observable fetching: boolean;
    @observable fetchParams: FetchParams;
    @observable subscription: Channel & {received(data: string): void} | null = null;
    @observable count: number;

    constructor(store: DataStore<Model>, channelName: string, fetchParams: FetchParams) {
        makeObservable(this);
        this.store = store;
        this.models = [];
        this.fetching = false;
        this.fetchParams = fetchParams;
        this.count = 0;

        this.fetch();
        this.subscription = this.setSubscription(channelName);
    }

    destructor() {
        consumer.subscriptions.remove(this.subscription);
    }

    @action
    async fetch() {
        runInAction(() => {
            this.fetching = true;
        })
        this.fetching = true
        const resp = await this.store.transportLayer.getAll(this.fetchParams);
        if (resp) {
            if (resp.models && isArrayLike(resp.models)) {
                runInAction(() => {
                    this.count = resp.count;
                    this.models = this.store.mapDataToIds(resp.models);
                })
            } else {
                console.error('not implemented')
            }
        }
        runInAction(() => {
            this.fetching = false;
        })
    }

    @action
    setModel(model: Model) {
        let m: Model[] = [];
        // We do not want to add models into the LatestModelsTable if were not at the most up to date data
        if(this.fetchParams.page !== 0 || this.fetchParams.filtered.length !== 0) { return; }

        if(this.models.length < (this.fetchParams.limit || 10)) {
            m = [model, ...this.models];
        } else {
            m = [model, ...this.models.slice(0,-1)];
        }
        runInAction(() => {
            this.models = m;
        })
    }

    setSubscription(channelName: string) {
        // Saving model because `this` is rebinded within the received callback
        const t = this;
        return consumer.subscriptions.create({
            channel: channelName
        }, {
            received(data: IncomingData) {
                const model = t.store.createModel(data.model);
                model.actionCableTimeStamp = data.timestamp || null;
                const id = model.getId().toString();
                if (data.action === "create") {
                    t.setModel(model);
                } else if (data.action === "update") {
                    const existingModel = t.store.data.get(id.toString())
                    if(!existingModel?.viewDirty) {
                        t.setModel(model);
                    }
                }
            }
        })
    }
}