import {attribute, DataStore, Model, model} from '@rx-suite/data';
import {t} from "@lingui/macro"
import {action} from "mobx";
import moment, {Moment} from "moment";
import {JobSide} from "./RequiredProcessStore";

export type RequiredProcessesToUpdate = {
    job_process_id: number;
    left: {
        capabilities: Array<number>;
    } | null;
    right: {
        capabilities: Array<number>;
    } | null;
};

type RejectReasonKey = 'noValidPath'
    | 'routedWhileWaitingOtherProcess'
    | 'markedDead'
    | 'noProcesses'
    | 'noProcessesAtNonStart'
    | 'manualReject'
    | 'processAborted'
    | 'timeSyncError'
    | 'returnJobToTray'
    | 'failedToSynchronize'
    | 'failFetchProcesses'
    | 'receivedStartInReject'
;

@model({
    transportLayer: "/api/orders",
    permissionTarget: "Order",
    routingInfo: {
        update: 'jobs/:id/edit'
    }
})

export class OrderModel extends Model {
    @attribute({
        label: t`Order Number`,
    })
    number: string = '';

    @attribute()
    job_numbers: {
        left: string;
        right: string;
    } = {
        left: '',
        right: ''
    };

    @attribute()
    required_process_ids: number[] = [];

    @attribute()
    scheduled_processes: Array<{ name: string, complete_at: string }> = [];

    @attribute()
    machine_schedule_ids: Array<number> = [];

    @attribute({
        label: t`Is being routed?`,
    })
    is_routing: boolean = false;

    @attribute({
        label: t`Reject reason`,
    })
    reject_reason: RejectReasonKey | null = null;

    @attribute({
        label: t`Reject description`,
    })
    reject_description: string = "";

    @attribute({
        label: t`Reject data`,
    })
    reject_data: any = null;

    @attribute({
        label: t`Reject time`,
    })
    reject_time: string = '';

    @attribute({
        label: t`Location`,
    })
    location: string | null = null;

    @attribute({
        label: t`Location time`,
    })
    location_time: Moment = moment();

    @attribute()
    rx_data_order_id: string | null = null;

    @action.bound
    isRejected = (): boolean => {
        return !!this.reject_reason;
    };

    @action.bound
    rejectDescription = (): string | null => {
        return buildRejectDescription(this.reject_reason, JSON.parse(this.reject_data), this.reject_description);
    };

    @action.bound
    completeUpTo = async (jobProcessRank: number): Promise<any> => {
        const response = await this.store.transportLayer.fetch(`/${this.getId()}/complete_up_to_process`, {
            method: "POST",
            headers: {'Content-Type': 'application/json'},
            body: JSON.stringify({
                job_process_rank: jobProcessRank
            })
        });

        return await response.json();
    };

    @action.bound
    resetDownTo = async (jobProcessRank: number): Promise<any> => {
        const response = await this.store.transportLayer.fetch(`/${this.getId()}/reset_down_to_process`, {
            method: "POST",
            headers: {'Content-Type': 'application/json'},
            body: JSON.stringify({
                job_process_rank: jobProcessRank
            })
        });

        return await response.json();
    };

    @action.bound
    reworkUpTo = async (jobProcessRank: number, side: JobSide): Promise<any> => {
        const response = await this.store.transportLayer.fetch(`/${this.getId()}/rework_up_to_process`, {
            method: "POST",
            headers: {'Content-Type': 'application/json'},
            body: JSON.stringify({
                job_process_rank: jobProcessRank,
                side: side
            })
        });

        return await response.json();
    };

    @action.bound
    updateRequiredProcesses = async (requiredProcessesData: Array<RequiredProcessesToUpdate>): Promise<any> => {
        const response = await this.store.transportLayer.fetch(`/${this.getId()}/update_required_processes`, {
            method: "POST",
            headers: {'Content-Type': 'application/json'},
            body: JSON.stringify({
                required_processes_data: requiredProcessesData
            })
        });

        return await response.json();
    };

    @action.bound
    setOrderMachineSchedule = async (machineIds: Array<string>): Promise<any> => {
        const response = await this.store.transportLayer.fetch(`/${this.getId()}/set_order_machine_schedule`, {
            method: "POST",
            headers: {'Content-Type': 'application/json'},
            body: JSON.stringify({
                machine_ids: machineIds
            })
        });

        return await response.json();
    };

    @action
    async toggleReject() {
        const response = await this.store.transportLayer.fetch(`/${this.getId()}/toggle_reject`, {
            method: "PUT",
            headers: {
                "Content-Type": "application/json"
            }
        });

        return await response.json();
    }

    @action.bound
    getStatus = async (): Promise<any> => {
        const response = await this.store.transportLayer.fetch(`/${this.getId()}/status`, {
            method: "GET",
            headers: {'Content-Type': 'application/json'}
        });

        const data: {status: string} = await response.json();
        return data.status;
    };

    constructor() {
        super();
        super.setup();
    }
}

export async function findOrderByOrderNumber(orderNumber: string) {
    const resp = await OrderStore.transportLayer.fetch(`/search/${orderNumber}`, {
        method: "GET"
    })

    return await resp.json();
}

export async function createOrder(orderNumber: string) {
    const resp = await OrderStore.transportLayer.fetch('', {
        method: "POST",
        headers: {
            "Content-Type": "application/json"
        },
        body: JSON.stringify({order_number: orderNumber})
    })

    return await resp.json();
}

export function buildRejectDescription(reason: RejectReasonKey | null, data: any, description?: string) {
    if (reason == null) {
        return null;
    }

    if (description != null && description.trim() != '') {
        // use legacy description when non-empty string
        return description;
    }

    const missingData = t`This job is rejected`;
    if (data == null) {
        return missingData;
    }

    // validate that reject_data has the data needed to build description
    const requiredData = {
        noValidPath: ['process', 'location'],
        routedWhileWaitingOtherProcess: ['process'],
        markedDead: ['location', 'visits'],
        noProcesses: [],
        noProcessesAtNonStart: [],
        manualReject: [],
        processAborted: ['process'], // 'comment' is optional here
        timeSyncError: ['machine', 'process', 'state'],
        returnJobToTray: [],
        failedToSynchronize: [],
        failFetchProcesses: [], // 'source' or 'reason' are optional here
        receivedStartInReject: []
    };

    for (const dataKey of requiredData[reason]) {
        if (data[dataKey] != null) {
            continue;
        }
        return missingData;
    }

    return {
        noValidPath: t`No valid path exists for this job to receive process ${data.process} from location ${data.location}.`,
        routedWhileWaitingOtherProcess: t`Job tried to be routed while waiting for process ${data.process} to complete.`,
        markedDead: t`Job has been marked dead (visited location ${data.location}: ${data.visits} times).`,
        noProcesses: t`Job received an empty list of required processes.`,
        noProcessesAtNonStart: t`Job entered the line without visiting a start point.`,
        manualReject: t`Job was manually rejected.`,
        processAborted: data.comment || t`Job process ${data.process} was aborted by the machine.`,
        timeSyncError: t`Job received ${data.process} - ${data.state} from ${data.machine} with a time difference of over an hour in the past. Please verify machine time and timezone settings.`,
        returnJobToTray: t`Job has completed rework. Please return lens to tray.`,
        failedToSynchronize: t`Failed to load the most recent job states for this job. Verify job data in Rx-Data.`,
        failFetchProcesses: data.reason || (data.source ? t`Job does not exist in the data source (${data.source}).` : t`Unknown Error`),
        receivedStartInReject: t`LMS data did not contain LMACRODLP.`
    }[reason];
}

export const OrderStore = OrderModel.store as DataStore<OrderModel>;