import {attribute, DataStore, Model, model} from '@rx-suite/data';
import * as yup from 'yup';
import { t } from "@lingui/macro"
import {computed} from "mobx";
import {string} from "yup";

type PLC_CONNECTION = 'mop' | 'opcua';
type ROUTING_LEVEL = 'easy' | 'cr' | 'finishing'
type OPCUA_VARIANT = 'modular' | 'ncc' | 'flex_link';
type LMS_TAG_SOURCE = 'lms' | 'rx-data' | 'none';
type LMS_RULE_MODE = 'length_based' | 'priority_based';

@model({
    transportLayer: "/api/settings",
    permissionTarget: "Setting",
    singleton: true,
    eager: true,
    routingInfo: {
        read: '/settings/:id/edit',
        update: '/settings/:id/edit'
    }
})

export class SettingModel extends Model {
    //read-only, from ENV
    @attribute({
        label: t`PLC connection`
    })
    connection_type: PLC_CONNECTION  = 'opcua';

    //read-only, from ENV
    @attribute({
        label: t`Routing Level`,
    })
    routing_level: ROUTING_LEVEL = 'easy';

    /* OPC UA ONLY settings */

    @computed
    get plcIsOpcua(): boolean {
        return this.connection_type === 'opcua';
    }

    @computed
    get variantRequiresBasePath(): boolean {
        return this.plcIsOpcua && (this.variant === 'ncc' || this.variant === 'flex_link');
    }

    @attribute({
        label: t`OPC UA server URL`,
        validation: string().when('plcIsOpcua',{
            is: true,
            then: schema => schema.required(t`OPC UA server URL is required field`),
            otherwise: schema => schema.nullable()
        })
    })
    opcua_server_url: null | string = null;


    @attribute({
        label: t`Device Root Path`,
        validation: string().when('variantRequiresBasePath',{
            is: true,
            then: schema => schema.required(t`Device Root Path is a required field`),
            otherwise: schema => schema.nullable()
        })
    })
    device_root_path: null | string = null;

    @attribute({
        label: t`Use Scanners from Rx-Scanners?`,
        validation: yup.boolean()
    })
    use_rx_scanners: boolean = false;

    //read-only, from ENV
    @attribute({
        label: t`Variant`,
    })
    variant: OPCUA_VARIANT = 'ncc';

    /* MOP ONLY settings */

    @attribute({
        label: t`MOP Request Data format`,
        validation: string().when('plcIsOpcua',{
            is: false,
            then: schema => schema.required(t`Data format is a required field`),
            otherwise: schema => schema.nullable()
        })
    })
    data_format: string = 'XML';

    /* Shared Settings */

    @attribute({
        label: t`Tag data source`,
        validation: yup.string()
    })
    tag_data_source: LMS_TAG_SOURCE = 'lms';

    @attribute({
        label: t`LMS Rule Mode`,
        validation: yup.string().nullable()
    })
    lms_rule_mode: LMS_RULE_MODE = 'priority_based';

    @attribute({
        label: t`Max job fetcher retry`,
        validation: yup.number().nullable()
    })
    rx_data_max_fetch_attempts: number = 3;

    @attribute({
        label: t`Job fetcher timeout (seconds)`,
        validation: yup.number().nullable()
    })
    rx_data_fetch_timeout: number = 10;

    @attribute({
        validation: yup.number().required(t`This field is required`)
    })
    job_stale_time: number = 0;

    @attribute({
        label: t`Use default processes if job fetcher fails?`,
        validation: yup.boolean()
    })
    use_default_processes_on_fail_fetch: boolean = false;

    @attribute({
        label: t`Post job data to Rx-Data?`,
        validation: yup.boolean().nullable()
    })
    post_job_data: boolean = false;

    @attribute({
        label: t`Mark job dead after how many visits to the same point?`,
        validation: yup.number().nullable()
    })
    mark_job_dead_after_x_visit: number = 1;

    @attribute({
        label: t`Use Loop Load Balancing`,
        validation: yup.boolean()
    })
    loop_load_balancing_enabled: boolean = false;

    @attribute({
        label: t`Launch Router on Service Start`,
        validation: yup.boolean()
    })
    launch_server_on_service_start: boolean = false;

    @attribute({
        label: t`Max Allowed Delayed Jobs`,
        validation: yup.number()
    })
    max_allowed_delayed_jobs: number = 2000;

    @attribute()
    use_rx_data: boolean = false;

    @attribute()
    use_rx_data_machines: boolean = false;

    @attribute({
        label: t`Enable Reference Process?`,
        validation: yup.boolean()
    })
    reference_process_enabled: boolean = false;

    @computed
    get routingModeIsFinishing(): boolean {
        return this.routing_level === 'finishing';
    }

    @computed
    get tagSourceIsRxData(): boolean {
        return this.tag_data_source === 'rx-data'
    }

    @computed
    get tagSourceIsLms(): boolean {
        return this.tag_data_source === 'lms'
    }

    constructor() {
        super();
        super.setup();
    }
}

export const SettingStore = SettingModel.store as DataStore<SettingModel>;
