import {attribute, consumer, DataStore, Model, model} from '@rx-suite/data';
import * as yup from 'yup';
import { t } from "@lingui/macro"
import {CapabilitiesModel} from "./CapabilityStore";
import {CapabilityMachineModel} from "./CapabilityMachineStore";
import {action, computed} from "mobx";
import {JobProcessModel} from "./JobProcessStore";
import moment, {Moment} from "moment";
import {MesMachineStateModel} from "./MesMachineStateStore";
import {useRouterOnline} from "../../hooks/useRouterOnline";
import {number} from "yup";

type MachineType = 'MachineVirtual' | 'MachineMes' | 'MachineReject' | 'MachineStart' | 'MachineExit';
type MachineForceState = 'online' | 'offline' | 'use_state';
type MachineApplyProcessTrigger = 'auto_finish' | 'auto_start' | 'use_job_state';

type mesMachineExternalAttributes = {
    state: string,
    time: Moment
}

class MachineStoreDataStore<T extends Model> extends DataStore<T> {
    @action
    subscribe_machine_state_update(ids: string[]) {
        if(this.subscription) { this.unsubscribe(); }

        // Saving store because `this` is rebinded within the received callback
        const store = this as MachineStoreDataStore<T>;
        store.subscription = consumer.subscriptions.create({
            channel: 'MachineStateUpdateChannel', ids: ids
        }, {
            received(data: {id: string, state: string, time: Moment}) {
                const existingModel = store.data.get(data.id.toString()) as unknown as MachineModel;

                if(!existingModel) { return; }

                existingModel.external_attributes.state = data.state
                existingModel.external_attributes.time = data.time

                store.setModel(existingModel as unknown as T, false);
            }
        })
    }
}

@model({
    eager: true,
    transportLayer: '/api/machines',
    permissionTarget: "Machine",
    routingInfo: {
        list: '/machines',
        update: '/machines/:id/edit',
        create: '/machines/create',
    },
    storeClass: MachineStoreDataStore
})

export class MachineModel extends Model {
    @attribute()
    component_id: number = 0;

    @attribute({
        label: t`Machine Name`,
        validation: yup.string().required("Name is a required field")
    })
    name: string = "";

    @attribute({
        label: t`Serial Number`
    })
    sn: string | null = null;

    @attribute()
    type: MachineType = 'MachineVirtual';

    @attribute({
        label: t`Force duration`,
        validation: yup.number().nullable()
    })
    force_duration: number = 0;

    @attribute({
        label: t`Process`,
        validation: number().when('processIsRequired',{
            is: true,
            then: schema => schema.typeError(t`Job Process is a required field`),
            otherwise: schema => schema.nullable()
        })
    })
    job_process_id: number | null = null;

    @attribute({
        label: t`Process`
    })
    job_process: JobProcessModel | null = null;

    @attribute({
        label: t`Capabilities`,
        validation: yup.array()
    })
    capabilities: CapabilitiesModel[] = []

    @attribute({
        label: t`Capabilities`,
        validation: yup.array()
    })
    capability_ids: number[] = []

    @attribute({})
    capabilities_to_add: CapabilitiesModel[] = [];

    @attribute({
        label: t`Forced state`,
        validation: yup.string()
    })
    forced_state: MachineForceState = 'online'; //Since machines are always created as VMs

    //MES

    @attribute({
        label: t`Update machine group?`,
        validation: yup.boolean()
    })
    update_machine_group: boolean = false;

    @attribute({
        label: t`Mes machine state ids`,
        validation: yup.array()
    })
    mes_machine_state_ids: object = [];

    @attribute({})
    mes_machine_states_to_add: MesMachineStateModel[] = [];

    @attribute({
        label: t`Send Aborted Jobs To`,
        validation: yup.string().nullable()
    })
    selected_reject: string = '';

    @attribute({
        label: t`Apply process change when?`,
        validation: yup.string()
    })
    apply_process_change_on : MachineApplyProcessTrigger = 'auto_finish';

    @attribute({
        label: t`Machine Capabilities`,
        validation: yup.array().nullable()
    })
    capability_machines : CapabilityMachineModel[] = [];

    @attribute({
        label: t`Allow Job Start?`,
        validation: yup.boolean()
    })
    allow_job_start: boolean = false;

    @attribute({
        label: t`Restart Job?`,
        validation: yup.boolean()
    })
    restart_jobs: boolean = false;

    @attribute({
        label: t`Is Reject Station?`,
    })
    is_reject: boolean = false;

    @attribute()
    online_states: string[] = [];

    @attribute()
    external_attributes: mesMachineExternalAttributes = {
        state: 'WAITINGFORJOBS',
        time: moment()
    }

    @action
    has_capability(capabilityId: number) {
        return this.capability_machines.some((capability_machine) => capability_machine.capability_id === capabilityId && capability_machine.enabled);
    }

    canDelete(): boolean {
        return !useRouterOnline();
    }

    @computed
    get processIsRequired(): boolean {
        return this.type == 'MachineVirtual' || this.type == 'MachineMes';
    }

    @computed
    get isMesMachine(): boolean {
        return this.type == 'MachineMes';
    }

    @action
    setMachineVirtual = async (): Promise<any> => {

        const resp = await MachineStore.transportLayer.fetch(`/${this.getId()}/set_machine_virtual`, {
            method: "PUT",
            headers: {
                "Content-Type": "application/json"
            }
        });

        return resp.json();
    }

    constructor() {
        super();
        super.setup();
    }
}

export const MachineStore = MachineModel.store as MachineStoreDataStore<MachineModel>;
