module RxDataJobs
  class JobStateJob < ApplicationJob
    #Can be queued as machine_job_create since we need a sequential job when creating a new job.
    queue_as :default

    def perform(job_params, machine_params, job_state_params, re_queued=false)
      return unless ensure_params(job_params, machine_params, job_state_params, re_queued)

      @job, @order = select_job_and_order_target

      if @re_queued
        if (@order.nil? || @job.nil?) || should_restart_order?
          @job, @order = create_order
        end

        return abort("order has no processes") unless @order.has_required_processes?
      else
        if @order.nil? || @job.nil?
          return requeue_sequential if @machine.allow_job_start?

          return abort("no job match for job number")
        end

        return requeue_sequential if should_restart_order?

        # protect against the race condition where the sequential queue freshly created the order but no processes exist yet (still fetching)
        return requeue_sequential unless @order.has_required_processes?
      end

      @required_process = @job.required_processes.find_by(job_process: @machine.job_process)
      return abort("process not required for side.") if @required_process.nil?

      if Router.instance.online && @order.is_routing?
        handle_router_cache_has_order
      else
        handle_router_cache_no_order
      end
    end

    private

    def ensure_params(job_params, machine_params, job_state_params, re_queued)
      @args = [job_params, machine_params, job_state_params, re_queued]

      @re_queued = re_queued
      @state = job_state_params[:state]&.upcase
      @job_number = job_params[:number]
      @side = job_state_params[:side]
      @machine_sn = machine_params[:sn]
      @time =  build_time(job_state_params[:time])
      @machine = Machine.find_by(sn: machine_params[:sn])

      return abort("No match for sn.") if @machine.nil?
      return abort("Machine has no process.") if @machine.job_process.nil?

      return abort("invalid state") if @state.nil?

      @comment = build_comment(job_state_params)
      @source = build_source(job_state_params)

      true
    end

    def select_job_and_order_target
      job = Job.find_by(number: @job_number, side: @side)

      return [job, job.order] unless job.nil?

      [nil, nil]
    end

    def create_order
      order = OrderRequest::OrderRequest.order_for_number(@job_number, 'new')
      order.send_router_create
      return select_job_and_order_target
    end

    def requeue_sequential
      # We need the job state to be processed sequentially, so move to a special one-worker queue
      @args[3] = true # set requeue to true
      RxDataJobs::JobStateJob.set(queue: :machine_job_create).perform_later(*@args)
    end

    def should_restart_order?
      @machine.restart_jobs? && @order.required_process_touched_gt_rank_exists?(@machine.job_process.rank)
    end

    def handle_router_cache_has_order
      if @time < @order.created_at
        # If the job state was created before the job exists, then the router would have ignored it.
        # This for cases where a machine starts a job. If it sends more than one state in quick succession, only the first one is registered.
        Router.instance.synchronize_model(@order) do
          save_process_change
        end
      else
        only_save_history
      end
    end

    def handle_router_cache_no_order
      if job_state_is_stale
        @order.reject(Order.REJECT_REASONS[:timeSyncError])
        only_save_history
        @order.save

        RxModule::WorkerBroadcast.broadcast_later(model: @order, action: :update)

        return
      end

      opposite_process = @required_process.opposite_side_process

      if opposite_process.nil? || !opposite_process.is_rework?
        save_process_change
      else
        if @required_process.finished?
          # Process should be frozen to finished - we keep the abort data for the history, but we update the timestamp to now for the finished
          # important note: we set the state to aborted time + 1 second so that future job state synchronizations will ignore the abort
          @required_process.validate_and_assign_state(RequiredProcess.STATE[:finished], @source, (@time + 1.second), nil)
          only_save_history
        else
          save_process_change
        end
      end
    end

    def build_comment(job_state_params)
      unless job_state_params[:comment].blank? || job_state_params[:scrap_reason].blank?
        return "#{job_state_params[:comment]} (scrap: #{job_state_params[:scrap_reason]})."
      end
      return job_state_params[:comment] unless job_state_params[:comment].blank?
      return job_state_params[:scrap_reason] unless job_state_params[:scrap_reason].blank?
      nil
    end

    def build_source(job_state_params)
      return @machine.name unless job_state_params[:substate] == 'AUTOFINISHED'

      "#{@machine.name} (AUTO)"
    end

    def build_time(time_string)
      return Time.zone.now if time_string.nil?
      Time.zone.parse(time_string)
    end

    def job_state_is_stale
      return (Time.zone.now - @time) >= 45.minutes
    end

    def only_save_history
      @required_process.build_history(@state, @source, @time, @comment)
      @required_process.save
    end

    def save_process_change
      @required_process.validate_and_assign_state_with_history(@state, @source, @time, @comment)
      @required_process.save

      RxModule::WorkerBroadcast.broadcast_later(model: @required_process, action: :update, channel_class: RequiredProcessChannel)
    end

    def abort(msg)
      Rails.logger.warn "Job State #{@state} (job: #{@job_number}, side: #{@side}, sn: #{@machine_sn}) IGNORED - #{msg}"
    end
  end
end