class MachineFetcher
  class << self
    # Machine status check a first time in the cache and returns the cache hit if there's one.
    # If there's none then we reload the entire cache so that hopefully we don't need to reload it again.
    # We don't reload it it on a missing machine basis because then we expand our number of calls to Rx-Data greatly.
    # Machines shouldn't be added willy nilly, so a fetch status all on first machine miss should optimistically ensure
    # The cache is valid for the subsequent calls.
    # If the machine is missing still after the second call, Rx-Data is probably down, return nil object.
    def machine_status(rx_data_id)
      status = Rails.cache.read([self, rx_data_id])

      if status.nil?
        reload_cache
      else
        return status
      end

      status = Rails.cache.read([self, rx_data_id])

      if status.nil?
        nil_machine_status_object
      else
        status
      end
    end

    def force_cache_reload?
      Setting.instance.use_rx_data_machines? && !Machine.where.not(rx_data_id: nil).exists?
    end

    def reload_cache(force_reload=false)
      machine_statuses = fetch_external(force_reload)

      machine_statuses = machine_statuses.select {|k ,v| v[:is_conveyor_machine]}
      machine_statuses.each do |sn, machine_data|
        machine_model = Machine.where(rx_data_id: machine_data[:machine_id]).first
        if machine_model.nil?
          # need to create new mes machine
          if component_already_exists?(machine_data[:name])
            Rails::logger.info "[Machine Fetcher] Cannot create machine with name #{machine_data[:name]}; Component already exists."
            next
          end

          machine_model = MachineMes.new_from_params({
                                                       sn: sn,
                                                       rx_data_id: machine_data[:machine_id],
                                                       name: machine_data[:name]
                                                     })
          machine_model.save!
        end

        ensure_valid_and_write(machine_data[:machine_id], machine_data.merge(sn: sn), machine_model)
      end
    end

    def state_update_from_external(params)
      machine = Machine.where(rx_data_id: params[:machine_id]).first
      ensure_valid_and_write(params[:machine_id], {
        status: params[:state],
        time: params[:time]
      }, machine)
    end

    private

    def ensure_valid_and_write(rx_data_id, machine_data, machine_model)
      return unless should_write_to_cache?(rx_data_id, machine_data[:time])

      Rails.cache.write([self, rx_data_id], machine_data, expires_in: 5.minutes)
      broadcast_front_end(machine_model, machine_data)
    end

    def should_write_to_cache?(rx_data_id, time)
      cache_entry = Rails.cache.read(rx_data_id)

      # These checks happen in an order for a reason:
      #1 If the cache entry is nil, we want to save to cache no matter what, to save the sn.
      return true if cache_entry.nil?

      #2 Time can be nil here if the Machine does not have a state yet, dont write to cache
      return false if time.nil?

      #3 If a machine got cached with no state (if #1 for example) then write cache
      return true if cache_entry[:time].nil?

      Time.parse(cache_entry[:time]) < Time.parse(time)
    end

    def component_already_exists?(name)
      !!Component.where(name: name).first
    end

    def fetch_external(force_reload)
      if(force_reload)
        Rails.cache.write([self,  'status'], fetch_machine_all_status, expires_in: 1.minutes)
      end

      # Protects against bunch calls to rx-data if caches miss over and over
      Rails.cache.fetch([self, 'status'], expires_in: 1.minutes) do
        fetch_machine_all_status
      end
    end

    def fetch_machine_all_status
      begin
        RxModule::Communicator.call_module_method('rx_data', 'api/machines/status_all', :get)
      rescue => err
        Rails.logger.error "[Rx-Data Machine Cache] Failed to load. #{err}"
        {}
      end
    end

    def broadcast_front_end(machine_model, machine_data)
      MachineStateUpdateChannel.broadcast_to(machine_model, {
        id: machine_model.id,
        state: machine_data[:status],
        time: machine_data[:time]
      })
    end

    def nil_machine_status_object
      {
        time: nil,
        status: nil
      }
    end
  end
end