module OrderRequest
  class OrderRequestExisting < OrderRequest
    include TimedConnection

    def execute(order_number)
      @order = Order.find_by_number(order_number)

      # If the order is going to be a reject, complete the request here
      if @order.nil?
        @order = Order.create_new(order_number)
        @order.reject(Order.REJECT_REASONS[:noProcessesAtNonStart])
      else
        ensure_order_state
      end
    end

    def ensure_order_state
      if @order.rejected?
        unreject_on = [Order.REJECT_REASONS[:failedToSynchronize]]

        if unreject_on.include?(@order.reject_reason)
          @order.unreject
        end
      end

      # Remove any process that are no longer obtainable on the routing graph
      routing_graph = RoutingGraph.instance
      unless routing_graph.not_initialized?
        process_ids = routing_graph.process_ids_offered
        @order.job_for_side('left').required_processes.where.not(job_process_id: process_ids).in_batches.destroy_all
        @order.job_for_side('right').required_processes.where.not(job_process_id: process_ids).in_batches.destroy_all
      end

      synchronize_order_data
    end

    def synchronize_order_data
      return unless Setting.instance.use_rx_data_jobs?
      return Rails::logger.warn "Order #{@order.number} has no external id - job states may not be up to date" if @order.rx_data_order_id.nil?

      resp = RxModule::Communicator.call_module_method(
        'rx_data',
        "/api/orders/#{@order.rx_data_order_id}/latest_routing_states",
        :get,
        nil,
        nil,
        timed_connection(30)
      )

      resp[:job_states].each do |job_state|
        # If the machine is not in use for routing, or has no process, ignore the job state
        machine = Machine.find_by(sn: job_state[:machine_sn])
        next if machine.nil? || machine.job_process.nil?

        job = Job.find_by(rx_data_job_id: job_state[:job_id])
        next if job.nil? || job.rx_data_job_id.nil?

        # If the process is no longer required, ignore the job state
        required_process = RequiredProcess.find_by(job_process_id: machine.job_process.id, job_id: job.id)
        next if required_process.nil?

        time = if job_state[:time].nil?
                 Time.zone.now
               else
                 Time.zone.parse(job_state[:time])
               end

        required_process.validate_and_assign_state(job_state[:state], machine.name, time, job_state[:comment])

        # We only want to create a history if we're actually changing the process's state
        if required_process.changed?
          required_process.build_history(job_state[:state].upcase, machine.name, time, job_state[:comment])
          required_process.save!
        end

        # XBA 2024-01-29 - this might no longer be required now that the router can always reconstruct the abort reason
=begin
        if required_process.aborted? && !@order.rejected?
          @order.reject(Order.REJECT_REASONS[:processAborted], Time.zone.now, {
            process: machine.job_process.name,
            comment: job_state[:comment],
            machine: machine.name
          })
        end
=end
      end
    rescue => err
      Rails::logger.warn "Error while synchronizing order #{@order.number} (rx-data id: #{@order.rx_data_order_id}): #{err}"
      @order.reject(Order.REJECT_REASONS[:failedToSynchronize])
    end
  end
end