module ProcessRequests
  class Lms < ProcessRequest
    class DcsConnectionNotConfigured < StandardError; end

    attr_accessor :source

    def initialize(job_number, post_to_data=false)
      super(job_number)
      @source = nil
      fetch

      post_order_to_rx_data if post_to_data
    end

    def fetch
      Rails::logger.info "Starting LMS tag request for job #{job_number}."

      get_data

      @lens_numbers = lens_numbers_from_tag_data

      @processes = JobProcessCalculator.instance.get_required_processes_from_tags(@tags, required_sides)

      raise NoProcesses unless @processes.length > 0
    end

    private

    def get_data
      lms_configs = LmsConfig.find_lms_configs_for_job_number(@job_number)

      tags_to_fetch = Tag.all.map(&:tag)

      lms_configs.each_with_index do |lms_config, index|
        raise DcsConnectionNotConfigured if lms_config.external_dcs_id.nil?

        resp = RxModule::Communicator.call_module_method_as_module("rx_link", "#{lms_config.external_url}/job_call", :post, {
          job_call: {
            job: @job_number,
            tags: tags_to_fetch
          }
        })

        if resp[:error]
          if index == (lms_configs.length-1) # Is last config, error out
            raise OrderDoesNotExist if resp[:error][:jobDoesNotExist]
            raise TimeoutWhileFetchingFromSource if resp[:error][:timeout]
            raise FailedToFetchFromSource
          end
        else
          @source = resp[:source]
          @tags = resp[:tags]
          break # Job found, end here
        end
      end
    end

    def lens_numbers_from_tag_data
      job_numbers = []
      job_numbers << {jobNumber: @job_number, lens: 'right'}.stringify_keys if do_right?
      job_numbers << {jobNumber: @job_number, lens: 'left'}.stringify_keys if do_left?
      job_numbers
    end

    concerning :PostToRxData do
      def post_order_to_rx_data
        Rails::logger.info "Posting Job Number #{@job_number} to Rx-Data."

        response = RxModule::Communicator.call_module_method('rx_data', "/api/orders/create_from_routing", :post, {
          mes: {
            evt: {
              id: @job_number,
              order: {
                name: @job_number,
                description: 'Created by Rx-Routing',
                data: format_tags,
                processes: format_processes
              }
            }
          }
        })

        @rx_data_order_id = response[:order_id]
        @rx_data_job_ids = response[:job_ids]
      end

      private

      def format_tags
        tag_data = {
          item: []
        }
        @tags.each do |side, tags|
          # Unlike processes, if a tag data is shared by both eyes, still send up separately (cannot use do 'B')
          next if tags.nil?
          tags.each do |tag, val|
            tag_data[:item] << {name: tag, lensside: side.to_s == "left" ? "L": "R", value: val}
          end
        end
        tag_data
      end

      def format_processes
        process_data = {
          process: []
        }
        @processes.each do |p|
          process = process_data[:process].detect { |v| v[:name] == p[:name]}
          # If the process is shared by both eye, set its side to B, otherwise use the side.
          if process.nil?
            process_data[:process] << {name: p[:name], eye: p[:side] == "left" ? "L": "R"}
          else
            process[:eye] = "B"
          end
        end
        process_data
      end
    end
  end
end
