module ProcessRequests
  class RxData < ProcessRequest
    include TimedConnection

    class IncompleteOrMissingData < StandardError; end

    def initialize(job_number)
      super(job_number)
      fetch
    end

    def source
      'Rx-Data'
    end

    def fetch
      Rails::logger.info "Starting Rx-Data tag request for job #{job_number}."

      get_data

      ensure_data

      @processes = JobProcessCalculator.instance.get_required_processes_from_tags(@tags, required_sides)

      raise NoProcesses unless @processes.length > 0
    end

    private

    def get_data
      max_attempt = Setting.instance.rx_data_max_fetch_attempts
      begin
        attempt ||= 1
        resp = RxModule::Communicator.call_module_method('rx_data', '/api/orders/routing_launch_data', :get, {job_number: @job_number}, nil, fetch_client)
      rescue Faraday::TimeoutError
        Rails::logger.warn "Timed out while fetching #{@job_number} from Rx-Data. Attempt #{attempt}/#{max_attempt}."
        attempt += 1
        if attempt <= max_attempt
          retry
        else
          raise TimeoutWhileFetchingFromSource
        end
      rescue => err
        Rails::logger.warn "Failed to fetch tags for job #{@job_number} from Rx-Data: #{err}"
        raise FailedToFetchFromSource
      end

      if resp[:error]
        raise OrderDoesNotExist if resp[:error][:orderDoesNotExist]
        raise FailedToFetchFromSource # Unknown error
      end

      if Setting.instance.reference_process_enabled?
        [:left, :right].each do |side|
          next if resp[:tag_data][side].nil?
          next if resp[:tag_data][side][:processes].nil?
          processes = resp[:tag_data][side][:processes].split(',')
          next unless is_reference_process?(processes)
          resp[:job_numbers] = resp[:job_numbers].select { |job_info| job_info[:lens].to_sym != side}
          resp[:tag_data][side] = nil
        end
      end

      @tags = resp[:tag_data]
      @lens_numbers = resp[:job_numbers].map { |job_number| job_number.stringify_keys }
      @rx_data_order_id = resp[:rx_data_order_id]
      @rx_data_job_ids = resp[:rx_data_job_ids]
      @job_active = resp[:active]
    end

    def fetch_client
      timed_connection(Setting.instance.rx_data_fetch_timeout)
    end

    def ensure_data
      raise IncompleteOrMissingData if @tags.blank?
      raise IncompleteOrMissingData if @lens_numbers.blank?
      raise IncompleteOrMissingData if @rx_data_order_id.nil? || @rx_data_job_ids.blank?
    end

    def is_reference_process?(processes)
      return false unless processes.length == 1
      %w[_reference _reference-rxnet].include?(processes[0])
    end
  end
end