class RouterClient
  include Singleton

  class RouterAlreadyStarted < StandardError; end

  def initialize
    @thread = nil
    @process_pid = nil
    @connection_mutex = Mutex.new # ensure router is not being start/stopped at the same time

    Kernel.at_exit do
      # Do not call stop, we want to by-pass the mutex
      begin
        terminate
      rescue
        # no-op, router might already be off
      end
      ensure_stop
    end
  end

  def start
    @connection_mutex.synchronize do
      Rails::logger.info("Starting Router Client")

      raise RouterAlreadyStarted unless @thread.nil?

      # Force the process cache to clear so next job call reloads the processes
      JobProcessCalculator.instance.clear_cache!

      @process_pid = spawn("router.exe --serverPort=#{ENV['ROUTER_SERVER_PORT']} --routingPort=#{ENV['MODULE_PORT']} --logLevel=verbose --environment=#{environment} --preloadRouteCache=#{!!ENV['PRELOAD_ROUTER_CACHE']}")

      # We do not want to wait for the process to complete (the router runs 'forever'), so monitor it via a thread
      @thread = Process.detach @process_pid

      # PID of the thread and process have to be saved to a file so the router can be forced killed on service start
      record_pid
    rescue => err
      Rails::logger.info("Failed to start Router Client: #{err}")
      ensure_stop
    end
  end

  def stop
    @connection_mutex.synchronize do
      Rails::logger.info("Stopping Router Client")
      begin
        terminate
      rescue
        # no-op, router might already be off
      end
      ensure_stop
    end
  end

  private

  def terminate
    RxModule::Communicator.call_module_method('router_api', '/terminate', :post)
  end

  def ensure_stop
    @thread.kill unless @thread.nil?
  rescue => err
    Rails::logger.info("Failed to stop Router Client: #{err}")
  ensure
    @thread = nil
    @process_pid = nil
  end

  def record_pid
    Dir.chdir(Rails.root) do
      File.open("./tmp/pids/router.pid", 'a') do |f|
        f.puts "#{@process_pid}"
      end
    end
    log_to_service("router pid - #{@process_pid} | thread pid #{@thread.pid}")
  end

  def log_to_service(msg)
    Dir.chdir(Rails.root) do
      File.open("./service-log.txt", 'a') do |f|
        f.puts msg
      end
    end
  end

  def environment
    return ENV['ROUTER_ENV'] unless ENV['ROUTER_ENV'].nil?

    "production"
  end
end
