class Device < ApplicationRecord
  include IsRouterResource
  include IsComponent
  include Synchronizable

  belongs_to :component, dependent: :destroy, required: true, autosave: true
  has_many :extra_requirements

  validates_associated :component, message: "'s name is already taken"

  class << self
    def new_from_params(params)
      if Setting.instance.use_opcua?
        OpcuaDevice.new_from_params(params)
      else
        MopDevice.new_from_params(params)
      end
    end
  end

  def assign_attributes_from_params(params)
    # Should be implemented by child class
    raise NotImplementedError
  end

  def inputs_with_scanners
    component.connectors.where.not(rx_scanner_reference: nil).or(component.connectors.where.not(opcua_scanner_reference: nil)).pluck(:name)
  end

  def has_control_actions?
    component.connectors.where.not(control_action_config: nil).count > 0
  end

  def should_broadcast?
    false
  end

  concerning :IsRouterResource do
    def synchronize_payload
      {
        method: :put,
        endpoint: '/device/update',
        data: as_router_data
      }
    end

    def as_router_data
      {
        identifier: component.name,
        connectors: component.connectors.map { |connector| connector.as_router_data },
        emulateJobStateSettings: if send_job_state_as.nil?
                                   nil
                                 else
                                   {
                                     sendAs: send_job_state_as,
                                     process: job_state_process,
                                     alwaysSendBoth: always_send_both_job_states
                                   }
                                 end
      }
    end
  end

  concerning :SendRouterHelpers do
    # Specialized message sends that don't fit in the typical synchronize schema

    def send_reset
      # Implemented by the inherited device class
      raise NotImplementedError
    end
  end
end
