class Job < ApplicationRecord
  include IsRouterResource
  include BatchDependentAssociations
  include TimedConnection

  belongs_to :order
  has_many :required_processes, dependent: :destroy, autosave: true

  enum side: [:left, :right]

  def should_broadcast?
    false
  end

  concerning :Sideable do
    def opposite_side
      order.jobs.select { |job| job.side != side }[0]
    end

    def required?
      required_processes.count > 0
    end
  end

  concerning :SetRequiredProcesses do
    def required_processes_update(required_processes_to_update)
      required_processes_to_update.each do |required_process_to_update|
        process_id = required_process_to_update[:job_process_id]
        required_process_data = required_process_to_update[side]

        if required_process_data.nil?
          remove_process(process_id)
        else
          update_or_add_process(process_id, required_process_data)
        end
      end
    end

    def remove_process(process_id)
      existing_required_process = required_process_for_process_id(process_id)
      unless existing_required_process.nil?
        existing_required_process.mark_for_destruction
      end
    end

    def add_process(process_id, required_process_data)
      return unless required_process_for_process_id(process_id).nil?

      req_process = required_processes.build({
                                               job_process_id: process_id,
                                               state: 'NOTSTARTED',
                                               capability_required_processes_attributes: required_process_data[:capabilities].map {|c_id| {
                                                 capability_id: c_id
                                               }},
                                               last_location_received_at: "manual",
                                               last_state_change_timestamp: Time.zone.now
                                             })
      req_process.required_process_state_histories.build({
                                                           required_process_id: req_process.id,
                                                           state: "NOTSTARTED",
                                                           source: "manual",
                                                           time: Time.zone.now
                                                         })
    end

    def update_or_add_process(process_id, required_process_data)
      existing_required_process = required_process_for_process_id(process_id)

      if existing_required_process.nil?
        add_process(process_id, required_process_data)
      else
        existing_required_process.set_capabilities(required_process_data[:capabilities])
      end
    end

    private

    def required_process_for_process_id(process_id)
      # Select must be used so that mark_for_destruction is applied on the instance INSIDE the model
      # Using find_by, causes a DIFFERENT instance to be loaded, so the mark_for_destruction gets lost when this model is saved
      required_processes.select {|required_process| required_process.job_process_id == process_id}[0]
    end
  end

  concerning :IsRouterResource do
    def as_router_data
      return nil unless required?

      {
        number: number,
        side: side,
        processes: required_processes.find_each.map { |reqProc| reqProc.as_router_data }
      }
    end
  end
end