class Machine < ApplicationRecord
  include IsRouterResource
  include IsComponent
  include Synchronizable

  class MachineDoesNotExistInRxData < StandardError; end
  class MachineConnectorsAlreadyExist < StandardError; end

  belongs_to :component, dependent: :destroy, required: true
  belongs_to :job_process, optional: true
  has_many :capability_machines, :dependent => :destroy, autosave: true
  has_many :capabilities, :through => :capability_machines
  has_many :order_machine_schedules, dependent: :delete_all
  has_many :scheduled_process_changes, dependent: :destroy
  has_many :reject_filters

  #INF stuff to be removed
  has_many :inf_jobs, class_name: 'Job'
  belongs_to :lms_config, required: false

  validates_associated :component, message: "'s name is already taken"

  # none is an already defined keyword for enums.
  enum forced_state: ['online', 'offline', 'use_state']

  class << self
    def machine_template_to_class
      {
        'MachineReject' => MachineReject,
        'MachineStart' => MachineStart,
        'MachineExit' => MachineExit,
        'MachineVirtual' => MachineVirtual,
      }
    end
  end

  def assign_attributes_from_params(params)
    unless job_process.nil?
      # machine process has changed, clear all capabilities
      if job_process_id != params[:job_process_id]
        capability_machines.each do |machine_capability|
          machine_capability.mark_for_destruction
        end
      end
    end

    # Machine that are not synced with rx-data can update their name
    if rx_data_id.nil?
      component.assign_attributes({
                                    name: params[:name]
                                  })
      component.save!
    end

    assign_attributes(
      job_process_id: params[:job_process_id],
      force_duration: params[:force_duration],
      forced_state: params[:forced_state]
    )

    # Should never have that many capabilities, fine to do to_a
    current_capability_ids = capability_machines.pluck(:capability_id)
    params[:capabilities_to_add].each do |capability_to_add|
      unless current_capability_ids.include? capability_to_add[:capability_id]
        capability_machines.build(capability_id: capability_to_add[:capability_id], enabled: true)
      end
    end
  end

  def ensure_order_machine_schedule(new_job_process_id)
    # clear order_machine_schedule entries related to this record if job_process_id changes
    if job_process_id != new_job_process_id
      order_machine_schedules.destroy_all
    end
  end

  def restart_jobs?
    restart_jobs
  end

  def process_rank
    job_process.rank
  end

  def is_reject
    job_process&.name == 'processed' && capabilities.where(name: 'rejected').first
  end

  def build_connectors
    raise MachineConnectorsAlreadyExist if component.connectors.length > 0

    machine_connectors = [
      {
        name: 'input',
        connector_type: 'input',
        state: 'online'
      },
      {
        name: 'output',
        connector_type: 'output',
        state: 'online'
      }
    ]

    machine_connectors.each do |machine_connector|
      component.connectors << MachineConnector.new(machine_connector)
    end
  end

  concerning :SharedProperties do
    def external_attributes
      {}
    end

    def online_states
      []
    end

    def mes_machine_state_ids
      []
    end

    def controller_name
      "MachinesController"
    end
  end

  concerning :IsRouterResource do
    def synchronize_payload
      {
        method: :put,
        endpoint: '/machine/update',
        data: as_router_data
      }
    end

    def as_router_data
      {
        identifier: component.name,
        type: type,
        forcedState: forced_state,
      }
    end

    private

    def process_router_data
      {
        process: process_as_router_data,
        applyProcessChangeOn: apply_process_change_on,
        processingDuration: force_duration,
      }
    end

    def process_as_router_data
      return nil if job_process.nil?

      {
        name: job_process.name,
        capabilities: capability_machines.joins(:capability).where(enabled: true).pluck(:name)
      }
    end
  end
end