class OpcuaDevice < Device
  belongs_to :device_type, optional: true
  has_many :device_routing_command_devices, :dependent => :destroy
  has_many :device_routing_commands, :through => :device_routing_command_devices
  has_many :device_timers, :dependent => :destroy, autosave: true

  class InvalidMachineType < StandardError; end

  class << self
    def new_from_params(params)
      device_type = DeviceType.find_by(id: params[:device_type_id])

      raise InvalidMachineType if device_type.nil?

      infer_params_from_type(params, device_type)

      component = Component.new
      component.device = OpcuaDevice.new
      component.device.device_type_id = device_type.id
      component.device.assign_attributes_from_params(params)

      device_type.get_unique_opcua_connectors.each do |connector|
        connector = OpcuaConnector.new({
                                         name: connector.name,
                                         variable: connector.variable,
                                         connector_type: connector.connector_type,
                                         priority: connector.priority,
                                         action_delay: connector.action_delay,
                                         state: 'online'
                                       })
        component.connectors << connector
      end

      device_type.device_timers.each do |timer|
        timer = DeviceTimer.new({
                                  variable: timer.variable,
                                  time: timer.time,
                                  description: timer.description,
                                })
        component.device.device_timers << timer
      end

      component.device.device_routing_command_devices.new(device_type.device_routing_command_ids.map {|drci| {device_routing_command_id: drci}})

      component.device
    end

    def infer_params_from_type(params, type)
      [:routing_retry_delay, :delay_on_command_complete].each do |key|
        params[key] = type[key] if params[key].nil?
      end
    end
  end

  def assign_attributes_from_params(params)
    component.name = params[:name]
    assign_attributes(
      routing_retry_delay: params[:routing_retry_delay],
      delay_on_command_complete: params[:delay_on_command_complete],
      send_job_state_as: params[:send_job_state_as],
      job_state_process: params[:job_state_process],
      always_send_both_job_states: params[:always_send_both_job_states]
    )
  end

  def controller_name
    "DevicesController"
  end

  concerning :IsRouterResource do
    def as_router_data
      base = super

      base.merge!(
        {
          type: device_type.name,
          routingRetryDelay: routing_retry_delay,
          delayOnCommandComplete: delay_on_command_complete,
          opcuaConstants: device_timers.map { |timer| timer.as_router_data},
          routingCommands: device_routing_command_devices.map { |command| command.as_router_data }
        }
      )
    end
  end

  concerning :SendRouterHelpers do
    # Specialized message sends that don't fit in the typical synchronize schema

    def send_reset
      Router.instance.send_message(:put, "#{router_device_endpoint}/reset", { device: component.name })
    end

    private

    def router_device_endpoint
      '/device'
    end
  end
end