class Order < ApplicationRecord
  include Synchronizable
  include TimedConnection

  class InvalidSide < StandardError; end

  has_many :jobs, dependent: :destroy, autosave: true
  has_many :order_histories, dependent: :delete_all
  has_many :order_machine_schedules, dependent: :delete_all, autosave: true
  has_many :scheduled_process_changes, dependent: :delete_all

  class << self
    def create_new(order_number)
      Order.ensure_does_not_exist(order_number)
      order = Order.new(number: order_number)
      order.jobs = [:left, :right].map {|side| Job.new({number: order_number, side: side})}
      order.order_histories.build(
        event_type: 'order_create',
        data: {}.to_json,
        time: Time.zone.now
      )
      order.save!
      order
    end

    def ensure_does_not_exist(order_number)
      old_order = Order.find_by(number: order_number)
      unless old_order.nil?
        Rails::logger.info "Order #{order_number} is already in use. Removing existing order data."
        old_order.ensure_destroy!
      end
    end

    def REJECT_REASONS
      {
        manual: 'manualReject',
        timeSyncError: 'timeSyncError',
        processAborted: 'processAborted',
        failedToSynchronize: 'failedToSynchronize',
        failFetchProcesses: 'failFetchProcesses',
        noProcessesAtNonStart: 'noProcessesAtNonStart',
        receivedStartInReject: 'receivedStartInReject'
      }
    end
  end

  def save_and_force_refresh!
    self.updated_at = Time.zone.now
    save!
  end

  def ensure_destroy!
    destroy!
    # Always notify router that delete was invoked
    send_router_delete
  end

  def scheduled_processes
    scheduled_process_changes.joins(machine: [:job_process]).pluck(:name, :complete_at).map{ |e| { name: e[0], complete_at: e[1] } }
  end

  def machine_schedule_ids
    order_machine_schedules.pluck(:machine_id)
  end

  concerning :IsRouterResource do
    def synchronize_payload
      {
        method: :put,
        endpoint: '/order/update',
        data: as_router_data
      }
    end

    def as_router_data
      {
        number: number,
        scheduling: order_machine_schedules.map { |schedule| schedule.as_router_data},
        reject: if reject_reason.nil?
                  nil
                else
                  {
                    key: reject_reason,
                    data: reject_data.nil? ? { } : JSON.parse(reject_data)
                  }
                end,
        jobs: {
          left: left_job.as_router_data,
          right: right_job.as_router_data
        }
      }
    end
  end

  concerning :Sideable do
    def job_for_side(side)
      return left_job if side == 'left'
      return right_job if side == 'right'
      raise InvalidSide
    end

    def job_numbers
      {
        left:  jobs.where(side: 'left').pluck(:number).first,
        right: jobs.where(side: 'right').pluck(:number).first
      }
    end

    private

    def left_job
      select_job 'left'
    end

    def right_job
      select_job 'right'
    end

    def select_job(side)
      # Select must be used so that mark_for_destruction is applied on the instance INSIDE the model
      # Using find_by, causes a DIFFERENT instance to be loaded, so the mark_for_destruction gets lost when this model is saved
      jobs.select { |job| job.side == side }[0]
    end
  end

  concerning :Reject do
    def rejected?
      !reject_reason.nil?
    end

    def reject(reason, time = Time.zone.now, data = {})
      order_histories.build({
                         event_type: :order_reject,
                         data: {
                           reject_reason: reason,
                           reject_data: data
                         }.to_json
                       })
      if time > reject_time
        assign_attributes({
                            reject_reason: reason,
                            reject_data: data.to_json,
                            reject_description: nil,
                            reject_time: time
                          })
      end
    end

    def unreject(time = Time.zone.now)
      order_histories.build({
                              event_type: :order_unreject,
                              data: {}.to_json
                            })
      if time > reject_time
        assign_attributes({
                          reject_reason: nil,
                          reject_data: nil,
                          reject_description: nil,
                          reject_time: time
                        })
      end
    end
  end

  concerning :ManualActions do
    def complete_processes_up_to(job_process_rank)
      ensure_order_state
      process_ids = JobProcess.lteq_rank(job_process_rank).pluck(:id)
      set_process_ids_to_state(process_ids, RequiredProcess.STATE[:finished])
    end

    def reset_processes_down_to(job_process_rank)
      ensure_order_state
      process_ids = JobProcess.gteq_rank(job_process_rank).pluck(:id)
      set_process_ids_to_state(process_ids, RequiredProcess.STATE[:notstarted])
    end

    def rework_processes_up_to(job_process_rank, side)
      ensure_order_state
      process_ids = JobProcess.lteq_rank(job_process_rank).pluck(:id)
      set_side_to_rework(process_ids, side)
    end


    ## TODO: We want to see if manual changes have been done on the order, and what those changes are.
    # Add something akin to the old process_snapshot to order_histories
    def update_required_processes(required_processes_to_update)
        ensure_order_state
        %w[left right].each do |side|
          current_job = job_for_side(side)
          current_job.required_processes_update(required_processes_to_update)
        end
    end

    def assign_order_machine_schedule(machine_ids)
      current_machine_ids = order_machine_schedules.map { |c| c.machine_id }
      to_remove = current_machine_ids - machine_ids
      to_add = machine_ids - current_machine_ids

      to_remove.each do |t|
        model_to_destroy = order_machine_schedules.select{ |order_machine_schedule| order_machine_schedule.machine_id == t }
        next if model_to_destroy.empty?
        model_to_destroy[0].mark_for_destruction
      end

      to_add.each do |t|
        order_machine_schedules.build({ machine_id: t })
      end
    end

    def apply_update_from_rx_net(required_process_data)
      Order.transaction do
        # NOTE: if only capabilities are modified, the front-end will not be updated
        update_required_processes(required_process_data)
        order_histories.build({
                                      event_type: :order_update,
                                      data: {
                                        location: location
                                      }.to_json,
                                      time: Time.zone.now
                                    })
        save_and_force_refresh!
      end
    end

    private

    def ensure_order_state
      unreject if rejected?
    end

    def set_process_ids_to_state(process_ids, state)
      current_time = Time.zone.now
      required_process_ids = RequiredProcess.where(job_id: job_ids).where(job_process_id: process_ids).pluck(:id)
      RequiredProcess.bulk_update(required_process_ids.map { |r_process_id| {
        id: r_process_id,
        state: state,
        last_state_change_timestamp: current_time,
        is_rework: false
      }})
      required_processes = RequiredProcess.where(id: required_process_ids)
      required_processes.each{ |r_process| r_process.broadcast_to_required_process_channel }
      required_process_entries = required_process_ids.map{|r_id| { required_process_id: r_id }}
      # TODO: use insert_all here
      RequiredProcessStateHistory.create(required_process_entries) do |r|
        r.state = state
        r.source = "manual"
        r.time = current_time
      end
      self.updated_at = current_time
    end
  end

  def set_side_to_rework(process_ids, side)
    # only one side of the order is allowed to be in rework
    opposite_side = side == 'left' ? 'right' : 'left'
    opposite_process_rework_count = RequiredProcess.joins(:job)
                                        .where(job_id: job_ids)
                                        .where(job_process_id: process_ids)
                                        .where(is_rework: true)
                                        .where(:jobs => {:side => opposite_side}).count
    return if opposite_process_rework_count > 0

    new_state = RequiredProcess.STATE[:notstarted]
    current_time = Time.zone.now
    required_process_ids = RequiredProcess.joins(:job)
                                          .where(job_id: job_ids)
                                          .where(job_process_id: process_ids)
                                          .where(is_rework: false)
                                          .where(:jobs => {:side => side}).pluck(:id)
    RequiredProcess.bulk_update(required_process_ids.map { |r_process_id| {
      id: r_process_id,
      state: new_state,
      is_rework: true,
      last_state_change_timestamp: current_time
    }})
    required_processes = RequiredProcess.where(id: required_process_ids)
    required_processes.each{ |r_process| r_process.broadcast_to_required_process_channel }
    required_process_entries = required_process_ids.map{|r_id| { required_process_id: r_id }}
    # TODO: use insert_all here
    RequiredProcessStateHistory.create(required_process_entries) do |r|
      r.state = new_state
      r.source = "rework"
      r.time = current_time
    end
    self.updated_at = current_time
    self.order_histories.create({
                                  event_type: :order_rework,
                                  data: {
                                    side: side
                                  }.to_json,
                                  time: Time.zone.now
                                })
  end


  concerning :SendRouterHelpers do
    # Specialized message sends that don't fit in the typical synchronize schema

    def send_router_create
      Router.instance.send_message(:post, "#{router_order_endpoint}/create", as_router_data)
    end

    def send_router_delete
      Router.instance.send_message(:delete, "#{router_order_endpoint}/delete", { number: number })
    end

    private

    def router_order_endpoint
      '/order'
    end

  end

  concerning :RequiredProcesses do
    def required_process_ids
      RequiredProcess.where(job_id: job_ids).pluck(:id).sort
    end

    def required_process_started?(job_process)
      [left_job, right_job].each do |job|
        process = job.required_processes.find_by(job_process_id: job_process.id)

        return false if process.nil? || !process.started?
      end
      true
    end

    def required_process_touched_gt_rank_exists?(rank)
      [left_job, right_job].each do |job|
        return true if job.required_processes.joins(:job_process).where("rank > #{rank}").where.not(state: RequiredProcess.STATE[:notstarted]).exists?
      end
      false
    end

    def has_required_processes?
      [left_job, right_job].each do |job|
        return true if job.required_processes.exists?
      end
      false
    end

    def has_process_named?(name)
      process_id = JobProcess.where(name: name).pluck(:id)
      RequiredProcess.where(job_id: job_ids).where(job_process_id: process_id).exists?
    end
  end

  def set_rx_data_identifiers(order_id, job_ids, lens_numbers)
    unless order_id.nil?
      self.rx_data_order_id = order_id
    end

    jobs.each do |job|
      rx_data_id = job_ids[job.side.to_sym]
      rx_data_number = lens_numbers.find {|data| data["lens"] == job.side}

      job.number = rx_data_number["jobNumber"] unless rx_data_number.nil?
      job.rx_data_job_id = rx_data_id unless rx_data_id.nil?
      job.save!
    end
  end

  def set_required_processes(req_processes_data)
    # Important note: we set the initial process time to 2 minute from now to solve a problem where job states that
    # start an order sometimes arrive a few seconds after the order was created (which mean they would normally be ignored).
    # While not ideal, it's the least intrusive method to deal with this problem.
    start_time = Time.now - 2.minute
    req_processes_data.each do |req_process_data|
      job_process = JobProcess.find_by_name(req_process_data[:name])
      next if job_process.nil?
      job = job_for_side(req_process_data[:side].to_s)
      req_process = job.required_processes.new(job_process: job_process)
      req_process.capabilities << job_process.capabilities.where(name: req_process_data[:capabilities])
      req_process.assign_state_with_history(RequiredProcess.STATE[:notstarted], 'manual', start_time, nil)
    end
    self.order_histories.create({
                                  event_type: :process_set,
                                  data: {}.to_json,
                                  time: Time.zone.now
                                })
  end

  # Rx-Module provides on_update and on_create, but .save! does not trigger on_update, so provide a custom hook
  after_save do |model|
    unless self.class.prevent_broadcast?
      ActionCable.server.broadcast "everything", {
        model: JSON.parse(to_json),
        class: model.class.name,
        action: :update
      }
    end
  end

  after_destroy do |model|
    ActionCable.server.broadcast "everything", {
      model: {id: model.id},
      class: model.broadcast_class_name,
      action: :destroy,
      timestamp: Time.now.to_i
    }
  end

  def should_broadcast?
    false
  end

  def to_json(*args)
    render_jbuilder_partial
  end
end