class RequiredProcess < ApplicationRecord
  include IsRouterResource

  belongs_to :job, touch: true
  belongs_to :job_process
  has_many :capability_required_processes, dependent: :delete_all
  has_many :capabilities, :through => :capability_required_processes
  has_many :required_process_state_histories, dependent: :delete_all
  accepts_nested_attributes_for :capability_required_processes

  class << self
    def STATE
      {
        started: 'STARTED',
        finished: 'FINISHED',
        aborted: 'ABORTED',
        notstarted: 'NOTSTARTED'
      }
    end

    def ensure_time_from_time_string(time)
      return Time.zone.now if time.nil?
      Time.zone.parse(time)
    end

  end


  def set_capabilities(capability_ids)
    existing_capabilities = capability_required_processes.pluck(:capability_id)
    to_add = capability_ids - existing_capabilities
    to_add.each do |capability_id|
      capability_required_processes.build({
                                            capability_id: capability_id
                                          }
      )
    end

    to_remove = existing_capabilities - capability_ids
    to_remove.each do |capability_id|
      # Select must be used so that mark_for_destruction is applied on the instance INSIDE the model
      # Using find_by, causes a DIFFERENT instance to be loaded, so the mark_for_destruction gets lost when this model is saved
      existing_capability = capability_required_processes.select { |capability_required_process| capability_required_process.capability_id == capability_id }[0]
      next if existing_capability.nil?
      existing_capability.mark_for_destruction
    end
  end


  concerning :Sideable do
    def side
      job.side
    end

    def opposite_side_process
      job.opposite_side.required_processes.where(job_process: job_process).first
    end
  end

  concerning :IsRouterResource do
    def as_router_data
      {
        id: id,
        process: {
          name: job_process.name,
          capabilities: capabilities.pluck(:name)
        },
        state: state,
        time: last_state_change_timestamp,
        lastSource: last_location_received_at,
        lastMachineComment: last_comment,
        isRework: is_rework
      }
    end
  end

  concerning :State do
    def notstarted?
      ensure_state == RequiredProcess.STATE[:notstarted]
    end

    def started?
      ensure_state == RequiredProcess.STATE[:started]
    end

    def aborted?
      ensure_state == RequiredProcess.STATE[:aborted]
    end

    def finished?
      ensure_state == RequiredProcess.STATE[:finished]
    end

    def assign_state_with_history(state, source, time, comment)
      handle_state_change([state, source, time, comment], true, false)
    end

    def validate_and_assign_state(state, source, time, comment)
      handle_state_change([state, source, time, comment], false, true)
    end

    def validate_and_assign_state_with_history(state, source, time, comment)
      handle_state_change([state, source, time, comment], true, true)
    end

    def build_history(state, source, time, comment)
      required_process_state_histories.build({
                                               state: state,
                                               source: source,
                                               time: time,
                                               comment: comment
                                             })
    end

    private

    def handle_state_change(args, with_history, with_validation)
      state, source, time, comment = args

      state = state.upcase
      time = ensure_present(time)

      build_history(state, source, time, comment) if with_history

      if with_validation
        return unless time_is_newer(time)
      end

      if state === 'ABORTED'
        state = 'FINISHED' if job_process.comment_matches_abort_override?(comment)
      end

      assign_attributes({
                          state: state,
                          last_state_change_timestamp: time,
                          last_comment: comment,
                          last_location_received_at: source
                        })
    end

    def time_is_newer(time)
      time > last_state_change_timestamp
    end

    def ensure_present(time)
      if time > Time.zone.now
        Time.zone.now
      else
        time
      end
    end

    def ensure_state
      state.upcase
    end
  end

  def post_current_as_job_state(machine, as_auto_finished=true)
    return if notstarted? # Rx-Data does not recognize the state 'not started'
    PostJobState.perform_later({
                               sn: machine.sn,
                               date: Time.at(self.last_state_change_timestamp).to_date,
                               time: Time.at(self.last_state_change_timestamp).to_time,
                               job_state: {
                                 job: self.job.number,
                                 lensside: self.side == 'left' ? 'L' : 'R',
                                 state: self.state,
                                 comment: self.last_comment,
                                 substate: as_auto_finished ? 'AUTOFINISHED' : nil,
                                 process: self.job_process.name
                               }
                             })
  end

  def broadcast_to_required_process_channel
    RequiredProcessChannel.broadcast_to(self, render_jbuilder_partial)
  end

  def should_broadcast?
    false
  end
end

