class Router < ApplicationRecord
  enum status: [:offline, :starting, :standby, :error, :online]

  class << self
    def instance
      Router.first_or_create!
    end
  end

  def start
    RouterClient.instance.start
    update(status: 'starting')
  end

  def stop
    Rails.logger.info 'Releasing all routing jobs...'
    #Note: we do it via update_all to be quick and not trigger a bunch of callbacks. On the front-end we'll force the job store to refresh
    Order.update_all(is_routing: false)

    RouterClient.instance.stop
    update(status: 'offline')
  end

  def ping
    Rails::logger.info "Sending ping router server"
    resp = RxModule::Communicator.call_module_method('router_api', '/status', :get)
    update(status: resp[:status])
  rescue
    update(status: 'offline')
  end

  def online
    status == "online"
  end

  def running
    status != "offline"
  end

  def broadcast_class_name
    'RouterServer'
  end

  concerning :Communicator do
    def synchronize_model(model)
      yield if block_given?
      payload = model.synchronize_payload
      send_message(payload[:method], payload[:endpoint], payload[:data])
    end

    def send_message(method, endpoint, data)
      return Rails::logger.info "Router is offline, skipping #{method} #{endpoint}" unless running

      payload = if data.nil?
                  nil
                else
                  {
                    data: data,
                    timestamp: Time.zone.now.to_s
                  }
                end

      start_time = Time.now
      RxModule::Communicator.call_module_method('router_api', endpoint, method, payload)

      Rails::logger.info " Request to endpoint #{endpoint} took #{((Time.now - start_time)*1000).round(0)}ms to complete"
    end
  end
end
