class Setting < ApplicationRecord
  include Synchronizable
  include IsRouterResource


  enum routing_level: [:easy, :cr, :finishing]
  enum connection_type: [:opcua, :mop]
  enum variant: [:ncc, :modular, :flex_link]

  enum lms_rule_mode: [:priority_based, :length_based]

  class MissingEnvironmentVar < StandardError; end
  class RxScannersNotInstalled < StandardError; end

  class << self
    def instance
      Setting.first_or_create! do |setting|
        # Only invoked if model does not exist

        raise MissingEnvironmentVar if ENV['PLC_CONTROL'].nil?
        raise MissingEnvironmentVar if ENV['RX_ROUTER_LEVEL'].nil?

        setting.connection_type = ENV['PLC_CONTROL'].downcase
        setting.routing_level = ENV['RX_ROUTER_LEVEL'].downcase

        if setting.use_opcua?
          raise MissingEnvironmentVar if ENV['OPCUA_VARIANT'].nil?
          setting.variant = ENV['OPCUA_VARIANT'].downcase
        end
      end
    end

    def register_push_targets
      begin
        Rails::logger.info  "Registering Rx-Data Push Targets..."
        RxModule::Communicator.call_module_method('rx_data', '/api/register_push_targets', :post, {push_target: {app: 'rx_router', machine: '/api/machines/change_from_external', machine_state: 'api/machines/state_from_external', job_state: '/api/jobs/rx_data_update'}})
        RxModule::Communicator.call_module_method('rx_data', '/api/register_push_targets', :post, {push_target: {app: 'router_api', machine_state: '/mes-data/machine-state', job_state: '/mes-data/job-state' }})
        Rails::logger.info  "Push Targets registered successfully!"
      rescue => err
        Rails::logger.warn "FAILED TO REGISTER PUSH TARGETS: #{err}"
      end
    end
  end

  concerning :SettingAccessors do
    def lms_tag_source?
      tag_data_source.downcase == 'lms'
    end

    def use_rx_scanners?
      return false unless use_opcua?
      use_rx_scanners
    end

    def use_rx_data?
      use_rx_data_jobs? || use_rx_data_machines?
    end

    def use_rx_data_machines?
      level = routing_level.downcase
      level == 'cr' || level == 'finishing'
    end

    def reference_process_enabled?
      reference_process_enabled
    end

    def use_rx_data_jobs?
      rx_data_tag_source? || post_job_data
    end

    def rx_data_tag_source?
      tag_data_source.downcase == 'rx-data'
    end

    def use_opcua?
      connection_type.downcase == 'opcua'
    end

    def use_machine_state?
      routing_level.downcase == 'cr'
    end

    def use_rx_scanners?
      return false unless use_opcua?
      use_rx_scanners
    end
  end

  concerning :RouterPayloads do
    def communicator_settings
      {
        routing: {
          port: ENV['MODULE_PORT']
        },
        scanner: if use_rx_scanners?
                   begin
                     rxScannersInfo = RxModule::Communicator::Hub.module_info('rx_scanners')
                     {
                       port: rxScannersInfo[:port]
                     }
                   rescue
                     raise RxScannersNotInstalled
                   end
                 else
                    nil
                 end
      }
    end

    def base_settings
      {
        engine: routing_level,
        plcDriver: connection_type
      }
    end

    def plc_settings
      if use_opcua?
        {
          serverUrl: opcua_server_url,
          variant: variant,
          deviceRootPath: device_root_path
        }
      else
        {
          dataFormat: data_format
        }
      end
    end

    def as_router_data
      {
        markOrderDeadAfter: mark_job_dead_after_x_visit,
        enableLoopLoadBalancing: loop_load_balancing_enabled,
      }
    end

    def synchronize_payload
      {
        method: :put,
        endpoint: '/settings/update',
        data: as_router_data
      }
    end
  end
end