class AddTablesForOrders < ActiveRecord::Migration[6.0]

  class Order < ActiveRecord::Base
    has_many :jobs
    has_many :order_histories, dependent: :delete_all
    belongs_to :old_job
  end

  class OrderMachineSchedule < ActiveRecord::Base
  end

  class OrderHistory < ActiveRecord::Base
    belongs_to :order
  end

  class JobHistory < ActiveRecord::Base
    belongs_to :job
  end

  class OldJob < ActiveRecord::Base
    has_one :order
  end

  class Job < ActiveRecord::Base
    has_many :required_processes, dependent: :destroy
    has_many :job_histories, dependent: :delete_all
    has_many :scheduled_process_changes, dependent: :delete_all
    belongs_to :inf_target_machine, class_name: 'Machine', required: false

    belongs_to :order

    enum side: [:left, :right]

    include RxModule::BulkUpdate
  end

  class RequiredProcess < ActiveRecord::Base
    belongs_to :job
    belongs_to :job_process
    has_many :capability_required_processes, dependent: :delete_all
    has_many :capabilities, :through => :capability_required_processes
    has_many :required_process_state_histories, dependent: :delete_all
  end

  def up
    # bulk update creates a temporary table called TEMP for the postgres adapter
    # When calling bulk_update more than once, it attempts create a duplicate table and fails
    # reconnect to the connection to fix this issue
    ActiveRecord::Base.connection.reconnect!

    Order.connection.schema_cache.clear!
    Order.reset_column_information

    OrderMachineSchedule.connection.schema_cache.clear!
    OrderMachineSchedule.reset_column_information

    OrderHistory.connection.schema_cache.clear!
    OrderHistory.reset_column_information

    JobHistory.connection.schema_cache.clear!
    JobHistory.reset_column_information

    Job.connection.schema_cache.clear!
    Job.reset_column_information

    RequiredProcess.connection.schema_cache.clear!
    RequiredProcess.reset_column_information

    create_table :orders do |t|
      t.string :number
      t.string :reject_reason
      t.string :reject_description
      t.text :reject_data
      t.string :location
      t.datetime :reject_time, :default => Time.zone.now
      t.datetime :location_time, :default => Time.zone.now
      t.boolean :is_routing, :default => false
      t.bigint :rx_data_order_id
      t.bigint :old_job_id

      t.timestamps
    end

    create_table :order_machine_schedules do |t|
      t.belongs_to :order
      t.belongs_to :machine

      t.timestamps
    end

    create_table :order_histories do |t|
      t.integer :event_type
      t.text :data
      t.datetime :time
      t.belongs_to :order
    end

    rename_table(:jobs, :old_jobs)

    create_table :jobs do |t|
      t.string :number, required: true
      t.integer :side, required: true
      t.belongs_to :order
      t.bigint :rx_data_job_id
      t.timestamps
    end

    change_table :scheduled_process_changes do |t|
      t.belongs_to :order
    end

    now_method = if DatabaseServer.mssql?
                   "GETDATE"
                 else
                   "NOW"
                 end

    # Make insert manager for old jobs into order table
    # insert 1000 orders at a time (insert limit on mssql)
    order_insert_arr = []
    OldJob.in_batches do |batch|
      order_insert_manager = Arel::InsertManager.new
      order_insert_manager.into(Order.arel_table)
      order_insert_manager.columns << Order.arel_table[:number]
      order_insert_manager.columns << Order.arel_table[:old_job_id]
      order_insert_manager.columns << Order.arel_table[:location]
      order_insert_manager.columns << Order.arel_table[:reject_reason]
      order_insert_manager.columns << Order.arel_table[:reject_description]
      order_insert_manager.columns << Order.arel_table[:reject_time]
      order_insert_manager.columns << Order.arel_table[:is_routing]
      order_insert_manager.columns << Order.arel_table[:location_time]
      order_insert_manager.columns << Order.arel_table[:updated_at]
      order_insert_manager.columns << Order.arel_table[:created_at]
      order_insert_manager.insert(batch.select(OldJob.arel_table[:number],
                                                    OldJob.arel_table[:id],
                                                    OldJob.arel_table[:last_known_location],
                                                    OldJob.arel_table[:reject_reason],
                                                    OldJob.arel_table[:reject_description],
                                                    Arel::Nodes::NamedFunction.new(now_method,[]).as("reject_time"),
                                                    OldJob.arel_table[:is_routing],
                                                    Arel::Nodes::NamedFunction.new(now_method,[]).as("location_time"),
                                                    OldJob.arel_table[:updated_at],
                                                    OldJob.arel_table[:created_at]).to_sql)

      order_insert_arr << order_insert_manager
    end

    full_order_query = order_insert_arr.map {|insert| insert.to_sql}.join('; ')
    ActiveRecord::Base.connection.execute(full_order_query)

    # Make insert manager for orders into jobs table
    # We're gonna add a job for both sides, then later when we check with job_numbers of old_jobs we'll delete non needed jobs
    # Rake task is gonna get rx_data_job_id so no need to transfer it here
    job_insert_arr = []
    Order.in_batches(of: 500) do |batch|
      jobs_insert_manager = Arel::InsertManager.new
      jobs_insert_manager.into(Job.arel_table)
      jobs_insert_manager.columns << Job.arel_table[:number]
      jobs_insert_manager.columns << Job.arel_table[:side]
      jobs_insert_manager.columns << Job.arel_table[:order_id]
      jobs_insert_manager.columns << Job.arel_table[:updated_at]
      jobs_insert_manager.columns << Job.arel_table[:created_at]
      # 500 Left + 500 Right at a time for total insert of 1000 (insert limit on mssql)
      order_select_for_left_jobs = batch.select(
        Order.arel_table[:number],
        0,
        Order.arel_table[:id],
        Order.arel_table[:updated_at],
        Order.arel_table[:created_at]
      )

      order_select_for_right_jobs = batch.select(
        Order.arel_table[:number],
        1,
        Order.arel_table[:id],
        Order.arel_table[:updated_at],
        Order.arel_table[:created_at]
      )
      order_select_union = '(' + order_select_for_left_jobs.to_sql + ') UNION (' + order_select_for_right_jobs.to_sql + ')'
      jobs_insert_manager.insert(order_select_union)

      job_insert_arr << jobs_insert_manager
    end

    full_job_query = job_insert_arr.map {|insert| insert.to_sql}.join('; ')
    ActiveRecord::Base.connection.execute(full_job_query)

    # Update scheduled_process_changes to use order instead of jobs
    # Arel doesnt have a way to make join statements within update queries, so resorting to raw sql string
    scheduled_p_c_update_query = if DatabaseServer.mssql?
                                   <<~SQL
update scheduled_process_changes set
order_id = o.id
from scheduled_process_changes spc
JOIN orders o on spc.job_id = o.old_job_id
                                   SQL
                                 else
                                   <<~SQL
update scheduled_process_changes 
set order_id = o.id
from orders o
where scheduled_process_changes.job_id = o.old_job_id
                                   SQL
end

    ActiveRecord::Base.connection.execute(scheduled_p_c_update_query)

    # To support where something in (subquery) we need to use Arel's SelectManager

    ['left', 'right'].each do |side|
      select_sided_required_processes = Arel::SelectManager
                                         .new
                                         .project(RequiredProcess.arel_table[:job_id])
                                         .from(RequiredProcess.arel_table)
                                         .where(RequiredProcess.arel_table[:side].eq(side))

      select_old_jobs_from_required_processes = Arel::SelectManager
                                                  .new
                                                  .project(OldJob.arel_table[:id])
                                                  .from(OldJob.arel_table)
                                                  .where(OldJob.arel_table[:id]
                                                               .in(select_sided_required_processes))

      select_orders_from_old_jobs = Arel::SelectManager
                                      .new
                                      .project(Order.arel_table[:id])
                                      .from(Order.arel_table)
                                      .where(Order.arel_table[:old_job_id]
                                                  .in(select_old_jobs_from_required_processes))

      select_new_job_from_order_and_side = Arel::SelectManager
                                             .new
                                             .project(Job.arel_table[:id], Order.arel_table[:old_job_id])
                                             .from(Job.arel_table)
                                             .join(Order.arel_table)
                                             .on(Order.arel_table[:id].eq(Job.arel_table[:order_id]))
                                             .where(Job.arel_table[:order_id]
                                                       .in(select_orders_from_old_jobs)
                                                       .and(Job.arel_table[:side].eq(side))
                                             )

      sql = <<~SQL
update required_processes 
set job_id = subquery.id
from (
  #{select_new_job_from_order_and_side.to_sql}
) as subquery
where side = #{ActiveRecord::Base.connection.quote(side)}
and job_id = subquery.old_job_id
      SQL

      ActiveRecord::Base.connection.execute(sql)
    end

    # Update job numbers from old_jobs.job_numbers
    jobs_to_update = []
    OldJob.joins(order: :jobs).distinct.find_each do |old_job|
      next if old_job.job_numbers.blank?

      job_data = JSON.parse(old_job.job_numbers)
      jobs_to_update.concat(job_data.map do |job_datum|
        side = job_datum['lens']
        next if side.nil?

        # inefficient?
        job_id = old_job.order.jobs.where(side: side).first.id
        {id: job_id, number: job_datum['jobNumber']}
      end)
    end

    Job.bulk_update(jobs_to_update)

    remove_column(:orders, :old_job_id)
    drop_table(:old_jobs)

    drop_table :job_histories
    drop_table :inf_requests
    drop_table :stashed_process_changes

    remove_column(:scheduled_process_changes, :job_id)
    remove_column(:job_processes, :inf_target_tag)

    rename_column :required_process_state_histories, :job_state_timestamp, :time

    change_table(:required_processes) do |t|
      t.remove :last_state_change_timestamp
      t.remove :side
      t.remove :force_to_machine

      t.datetime :last_state_change_timestamp, default: Time.zone.now
    end

    # NOTE: Need to reset cache so that last_state_change_timestamp is seen as datetime, not integer
    RequiredProcess.connection.schema_cache.clear!
    RequiredProcess.reset_column_information

    execute("update required_processes set last_state_change_timestamp='#{Time.zone.now.to_datetime.utc.iso8601}'")
  end

  def down
    raise ActiveRecord::IrreversibleMigration
  end
end
